import { isServer } from "solid-js/web";
import { ServerError, refetchRouteData } from "solid-start";

export const catchAuthErrors = async <T>(
  fn: () => T,
): Promise<Awaited<T> | undefined> => {
  try {
    return await fn();
  } catch (err) {
    if (isServer) {
      return;
    }

    if (!(err instanceof ServerError) || err.status != 401) {
      throw err;
    }

    console.error(err);

    // We refetch the auth related data once, it will transition to the login screen if the cookie is really missing
    if (refetching) {
      return;
    }
    // Running the auth refetch non-blocking, independently from the catchAuthErrors caller
    (async () => {
      refetching = true;
      await refetchRouteData(["auth"]);
      refetching = false;
    })();
  }
};
export let refetching = false;
