import {
  Component,
  lazy,
  Match,
  onMount,
  Show,
  Suspense,
  Switch,
  useContext,
} from "solid-js";
import { useAuth } from "nitropage";
import { Meta, Route, Routes, Title, useRouteData } from "solid-start";
import { StartContainer } from "./util";
import Welcome from "./welcome";
import Login from "./login";
import Dashboard from "../dashboad";
import { adminRouteData } from "./routeData";
import "../../server/rpc";
import { ConfigFn } from "../../config";
import { createResource } from "solid-js";
import { isServer } from "solid-js/web";
import "@fontsource/jost/700-italic.css";
import "@fontsource/inter/100.css";
import "@fontsource/inter/200.css";
import "@fontsource/inter/300.css";
import "@fontsource/inter/400.css";
import "@fontsource/inter/500.css";
import "@fontsource/inter/600.css";
import "@fontsource/inter/700.css";
import "@fontsource/inter/800.css";
import "@fontsource/inter/900.css";
import "../../css/admin.css";
import Project from "../dashboad/project";
import Settings from "../dashboad/project/settings";
import Pages from "../dashboad/project/pages";
import Projects from "../dashboad/projects";
import { Region } from "../common/toast";
import { ConfigContext } from "../context/page";

const Editor = lazy(() => import("../editor"));
const EditorFrame = lazy(() => import("../editor/frame"));

const LoginWrapper = function () {
  return (
    <StartContainer>
      <Title>Nitropage - Login</Title>
      <Login />
    </StartContainer>
  );
};

const AdminRouter: Component = function (p) {
  return (
    <Routes>
      <Route path="/" component={Dashboard}>
        <Route path="/" component={Projects} />
        <Route path="/project/:id" component={Project}>
          <Route path="/" component={Pages} />
          <Route path="/settings" component={Settings} />
        </Route>
      </Route>
      <Route
        path="/page/:id"
        component={() => {
          return (
            <>
              <Title>Nitropage - Admin</Title>
              <Editor />
            </>
          );
        }}
      />
      <Route
        path="/frame"
        component={() => {
          return (
            <>
              <EditorFrame />
            </>
          );
        }}
      />
      <Route path="/*" component={() => <div>404</div>} />
    </Routes>
  );
};

declare global {
  var npBlueprintsLoaded: boolean | Promise<any>;
}

const Admin: Component<{ config: ConfigFn }> = function (p) {
  const data = useRouteData<() => ReturnType<typeof adminRouteData>>();
  const auth = useAuth();
  const [config, setConfig] = useContext(ConfigContext)!;

  // Preload blueprints on the server
  const [preloadBlueprints] = createResource(async function () {
    if (!isServer) {
      return true;
    }

    if (global.npBlueprintsLoaded) {
      return global.npBlueprintsLoaded;
    }

    global.npBlueprintsLoaded = p.config({
      config,
      setConfig,
      admin: true,
    });

    await global.npBlueprintsLoaded;

    // In production: all the blueprints will be preloaded on the server once
    // In development: the blueprints will be reloaded on the server on every admin request
    // The reason for this difference is, so that new server$ functions can be registered during development
    global.npBlueprintsLoaded = import.meta.env.PROD;

    return true;
  });

  data.editorPage();
  data.editorPageRevision();

  onMount(function () {
    p.config({
      config,
      setConfig,
      admin: true,
    });

    // Removes "live" page styles
    // This is needed if the admin page is accessed via clientside navigation
    for (const el of document.querySelectorAll(".nitro-page-style")) {
      el.remove();
    }
  });

  data.showWelcome();
  data.auth();

  return (
    <>
      <Meta name="robots" content="noindex, follow" />
      <Show when={preloadBlueprints()}>
        <Region />
        <Suspense>
          <Switch>
            <Match when={data.showWelcome()}>
              <Title>Nitropage - Welcome</Title>
              <Welcome />
            </Match>
            <Match when={!data.showWelcome() && !auth()?.id}>
              <LoginWrapper />
            </Match>
            <Match when={!data.showWelcome() && auth()?.id}>
              <AdminRouter />
            </Match>
          </Switch>
        </Suspense>
      </Show>
    </>
  );
};

export default Admin;
