import { Component, createSignal, Show } from "solid-js";
import { FooterButton, FormLayout, FormLogo, StyleContainer } from "./util";
import { useSigninAction } from "nitropage";
import { css } from "nitropage/css";
import { ButtonSpinner, ConfirmButton } from "../common/buttons";
import { demoEnabled } from "../../util";
import { createForm, required } from "@modular-forms/solid";
import { Input } from "../common/inputs";

type LoginForm = {
  username: string;
  password: string;
};

const Login: Component = function () {
  const [loginForm, { Form, Field }] = createForm<LoginForm>();
  const [signinState, signin] = useSigninAction();
  const [actionResult, setActionResult] = createSignal();

  return (
    <StyleContainer>
      <FormLogo />
      <Form
        onSubmit={async (data) => {
          if (signinState.pending) {
            return;
          }

          const result = await signin({
            username: data.username,
            password: data.password,
          });

          setActionResult(result);
        }}
      >
        <FormLayout>
          <Field
            name="username"
            validate={[required("Please enter your username.")]}
          >
            {(field, props) => (
              <Input
                {...props}
                error={field.error}
                label="Username / Email"
                placeholder="Username / Email"
                theme="lg"
                value={demoEnabled() ? "admin" : undefined}
                required
                autofocus
              />
            )}
          </Field>
          <Field
            name="password"
            validate={[required("Please enter your password.")]}
          >
            {(field, props) => (
              <Input
                {...props}
                error={field.error}
                label="Password"
                placeholder="Password"
                theme="lg"
                value={demoEnabled() ? "1234" : undefined}
                required
                type="password"
              />
            )}
          </Field>
        </FormLayout>
        <FooterButton>
          <ConfirmButton type="submit">
            <ButtonSpinner show={signinState.pending}>Sign in</ButtonSpinner>
          </ConfirmButton>
        </FooterButton>
      </Form>
      <Show when={actionResult() === false}>
        <div
          class={css({
            color: "#e11d48",
            textAlign: "center",
            marginTop: "0.5rem",
          })}
        >
          Please check your signin details.
        </div>
      </Show>
    </StyleContainer>
  );
};

export default Login;
