import { createAuthData } from "nitropage";
import server$, { redirect } from "solid-start/server";
import { createRouteData, RouteDataArgs } from "solid-start";
import { prisma } from "../../prisma";
import { createAuthRpc, authData } from "nitropage/server";
import { catchAuthErrors } from "../../admin/auth";

const getCount = server$(async () => {
  const users = await prisma.nitroUser.count();
  return users <= 0;
});

const getPage = server$(async (pageId: string) => {
  await authData(server$.request, { error: "throw" });
  return await prisma.nitroPage.findUnique({
    include: {
      project: true,
      publishedRevision: true,
      revisions: { orderBy: { savedAt: "desc" } },
    },
    where: { id: pageId },
  });
});

const getLatestPageRevision = server$(async (pageId: string) => {
  await authData(server$.request, { error: "throw" });
  const revision = await prisma.nitroPageRevision.findFirst({
    where: {
      page: { id: pageId },
    },
    orderBy: {
      updatedAt: "desc",
    },
  });

  return revision;
});

const getPageIdFromPathname = function (pathname: string) {
  if (!pathname.startsWith("/admin/page/")) {
    return;
  }

  return pathname.split("/")[3];
};

export const adminRouteData = function ({
  params,
  data,
  location,
}: RouteDataArgs) {
  const auth = createAuthData(server$(createAuthRpc()));
  return {
    showWelcome: createRouteData(() => getCount(), { key: "showWelcome" }),
    ...auth,
    editorPage: createRouteData(
      async ([_, pathname]) => {
        const pageId = getPageIdFromPathname(pathname!);
        if (!pageId) {
          return;
        }

        const page = await catchAuthErrors(() => getPage(pageId));
        if (!page) {
          throw redirect("/admin");
        }
        return page;
      },
      { key: () => ["editorPage", location.pathname] },
    ),
    editorPageRevision: createRouteData(
      async ([_, pathname]) => {
        const pageId = getPageIdFromPathname(pathname!);
        if (!pageId) {
          return;
        }

        const revision = await catchAuthErrors(() =>
          getLatestPageRevision(pageId),
        );

        return revision;
      },
      {
        key: () => ["editorPageRevision", location.pathname],
      },
    ),
  };
};
