import { Component } from "solid-js";
import { useSignupAction } from "nitropage";
import { css } from "nitropage/css";
import { ButtonSpinner, ConfirmButton } from "../common/buttons";
import {
  FooterButton,
  FormLayout,
  StartContainer,
  FormLogo,
  StyleContainer,
} from "./util";
import { refetchRouteData } from "solid-start";
import {
  createForm,
  required,
  email,
  custom,
  getValue,
} from "@modular-forms/solid";
import { Input } from "../common/inputs";

type SignupForm = {
  username: string;
  email: string;
  password: string;
  password_confirm: string;
};

const Welcome: Component = function () {
  const [signupForm, { Form, Field }] = createForm<SignupForm>();
  const [signupState, signup] = useSignupAction();

  return (
    <StyleContainer>
      <StartContainer>
        <FormLogo />
        <div
          class={css({
            marginBottom: "2rem",
            fontSize: "1.15rem",
            fontWeight: "500",
            color: "rgb(71 85 105)",
          })}
        >
          Welcome! Get rockin by creating a new user for the administration.
        </div>
        <Form
          onSubmit={async (data) => {
            if (signupState.pending) {
              return;
            }
            await signup({
              username: data.username,
              password: data.password,
              email: data.email,
            });
            await refetchRouteData("showWelcome");
          }}
        >
          <FormLayout>
            <Field
              name="username"
              validate={[required("Please enter your username.")]}
            >
              {(field, props) => (
                <Input
                  {...props}
                  name="username"
                  label="Username"
                  placeholder="Username"
                  error={field.error}
                  value={"admin"}
                  theme="lg"
                  required
                />
              )}
            </Field>
            <Field
              name="email"
              validate={[
                required("Please enter your email address."),
                email("Please enter a proper email address."),
              ]}
            >
              {(field, props) => (
                <Input
                  {...props}
                  type="email"
                  name="email"
                  label="Email"
                  placeholder="Email"
                  error={field.error}
                  theme="lg"
                  required
                />
              )}
            </Field>
            <Field
              name="password"
              validate={[required("Please enter your password.")]}
            >
              {(field, props) => (
                <Input
                  {...props}
                  type="password"
                  label="Password"
                  placeholder="Password"
                  error={field.error}
                  theme="lg"
                  required
                />
              )}
            </Field>
            <Field
              name="password_confirm"
              validate={[
                required("Please enter your password."),
                custom(
                  (v) => getValue(signupForm, "password") === v,
                  "Both passwords have to correspond.",
                ),
              ]}
            >
              {(field, props) => (
                <Input
                  {...props}
                  type="password"
                  label="Confirm password"
                  placeholder="Confirm password"
                  error={field.error}
                  theme="lg"
                  required
                />
              )}
            </Field>
          </FormLayout>
          <FooterButton>
            <ConfirmButton type="submit">
              <ButtonSpinner show={signupState.pending}>Lets go!</ButtonSpinner>
            </ConfirmButton>
          </FooterButton>
        </Form>
      </StartContainer>
    </StyleContainer>
  );
};

export default Welcome;
