import {
  ParentComponent,
  JSX,
  splitProps,
  mergeProps,
  ValidComponent,
  ComponentProps,
  FlowProps,
  FlowComponent,
  Show,
} from "solid-js";
import { css, cssMemo, IStyle } from "../../css/fela";
import { Button } from "@kobalte/core";
import { LoaderIcon } from "./icons";
import clsx from "clsx";

export const LargeSelectButton: ParentComponent<
  { active?: boolean } & JSX.ButtonHTMLAttributes<HTMLButtonElement>
> = (p) => {
  const [props, rest] = splitProps(p, ["active"]);
  const style = cssMemo(() => {
    const hoverStyle: IStyle = {
      "--tw-ring-offset-shadow":
        "var(--tw-ring-inset) 0 0 0 var(--tw-ring-offset-width) var(--tw-ring-offset-color)",
      "--tw-ring-shadow":
        "var(--tw-ring-inset) 0 0 0 calc(2px + var(--tw-ring-offset-width)) var(--tw-ring-color)",
      boxShadow:
        "var(--tw-ring-offset-shadow), var(--tw-ring-shadow), var(--tw-shadow, 0 0 #0000)",
    };

    const activeStyle: IStyle = {
      ...hoverStyle,
      "--tw-ring-opacity": 1,
      "--tw-ring-color": "var(--np-accent-400)",
      "--tw-bg-opacity": "1 !important",
      backgroundColor: "rgb(226 232 240 / var(--tw-bg-opacity)) !important",
    };

    return {
      "--tw-text-opacity": 1,
      color: "rgb(30 41 59 / var(--tw-text-opacity))",
      textAlign: "center",
      padding: "0.75rem 0",
      borderRadius: "0.5rem",
      borderWidth: "1px",
      "--tw-border-opacity": 1,
      borderColor: "rgb(203 213 225 / var(--tw-border-opacity))",
      "--tw-bg-opacity": 1,
      backgroundColor: "rgb(241 245 249 / var(--tw-bg-opacity))",
      "--tw-ring-opacity": 1,
      "--tw-ring-color": "var(--np-accent-300)",
      "--tw-ring-offset-width": "2px",
      ":hover": !rest.disabled ? hoverStyle : undefined,
      "&.active": activeStyle,
    };
  });

  return (
    <button class={clsx(style(), props.active && "active")} {...rest}>
      {p.children}
    </button>
  );
};

export const ConfirmButton: ParentComponent<
  { active?: boolean } & JSX.ButtonHTMLAttributes<HTMLButtonElement>
> = (p) => {
  const [props, rest] = splitProps(p, ["active"]);
  const style = cssMemo(() => {
    const activeStyle: IStyle = {
      "--tw-ring-offset-shadow":
        "var(--tw-ring-inset) 0 0 0 var(--tw-ring-offset-width) var(--tw-ring-offset-color)",
      "--tw-ring-shadow":
        "var(--tw-ring-inset) 0 0 0 calc(4px + var(--tw-ring-offset-width)) var(--tw-ring-color)",
      boxShadow:
        "var(--tw-ring-offset-shadow), var(--tw-ring-shadow), var(--tw-shadow, 0 0 #0000)",
      outline: "none",
    };

    return {
      flex: "1 1 0%",
      padding: "0.75rem 1rem",
      borderRadius: "0.25rem",
      "--tw-bg-opacity": 1,
      "--tw-text-opacity": 1,
      color: "rgb(255 255 255 / var(--tw-text-opacity))",
      transition: "all 150ms cubic-bezier(0.1, 0, 0.2, 1)",
      backgroundColor: "var(--np-accent-alt-950)",
      "--tw-shadow":
        "0 1px 3px 0 rgb(0 0 0 / 0.1), 0 1px 2px -1px rgb(0 0 0 / 0.1)",
      "--tw-shadow-colored":
        "0 1px 3px 0 var(--tw-shadow-color), 0 1px 2px -1px var(--tw-shadow-color)",
      boxShadow:
        "var(--tw-ring-offset-shadow, 0 0 #0000), var(--tw-ring-shadow, 0 0 #0000), var(--tw-shadow)",
      "--tw-ring-offset-width": "2px",
      "--tw-ring-opacity": 1,
      "--tw-ring-color": "var(--np-accent-400)",
      ":hover,:focus": !rest.disabled ? activeStyle : undefined,
      ":active": !rest.disabled
        ? {
            backgroundColor: "rgb(71 85 105)",
          }
        : undefined,
      "&.active": activeStyle,
    };
  });
  return (
    <button class={clsx(style(), props.active && "active")} {...rest}>
      {p.children}
    </button>
  );
};

export const SmallButton = function <
  T extends ValidComponent = "button",
  P extends ComponentProps<T> = ComponentProps<T>,
>(
  ps: FlowProps<{
    active?: boolean;
    theme?: "primary" | "secondary" | "transparent";
    component?: T;
  }> & { [K in keyof P]: P[K] },
) {
  const p = mergeProps({ theme: "secondary" }, ps);
  const [props, rest] = splitProps(p, ["theme", "active"]);
  const style = cssMemo(() => {
    const active: IStyle = {
      outline: "none",
      boxShadow: "0 0 0 1px white, 0 0 0 3px var(--np-accent-400)",
    };

    return {
      display: "flex",
      alignItems: "center",
      justifyContent: "center",
      gap: "0.5rem",
      borderRadius: "0.25rem",
      border: { primary: "1px solid #6b7280", secondary: "1px solid #9ca3af" }[
        props.theme
      ],
      transition: "all 150ms cubic-bezier(0.1, 0, 0.2, 1)",
      backgroundColor: {
        secondary: "#e5e7eb",
        primary: "#6b7280",
        transparent: undefined,
      }[props.theme],
      boxShadow:
        props.theme === "transparent"
          ? undefined
          : "0 1px 2px 0 rgb(0 0 0 / 0.05)",
      padding: "0.25rem 0.75rem",
      lineHeight: 1,
      color: { primary: "white", secondary: "#374151" }[props.theme],
      ":hover,:focus": !ps.disabled ? active : undefined,
      ":active": {
        color: { secondary: "#1f2937", primary: "white" }[props.theme],
        backgroundColor: { secondary: "#d1d5db", primary: "#4b5563" }[
          props.theme
        ],
      },
      "&.active": active,
      "&:disabled": {
        opacity: 0.5,
      },
    };
  });
  return (
    <Button.Root class={clsx(style(), props.active && "active")} {...rest}>
      {p.children}
    </Button.Root>
  );
};

export const ButtonSpinner: FlowComponent<{ show: boolean }> = function (p) {
  return (
    <div
      class={css({
        display: "flex",
        gap: "0.25rem",
        alignItems: "center",
        justifyContent: "center",
        lineHeight: 1,
      })}
    >
      {p.children}{" "}
      <Show when={p.show}>
        <LoaderIcon
          class={css({
            animation: "np-spin 1s linear infinite",
          })}
        />
      </Show>
    </div>
  );
};
