import { Checkbox as KCheckbox } from "@kobalte/core";
import { children, ParentComponent, Show } from "solid-js";
import { css } from "../../css/fela";
import { CheckIcon } from "./icons";

const Checkbox: ParentComponent<{
  title?: string;
  state: boolean;
  setState: (v: boolean) => void;
}> = function (props) {
  const child_ = children(function () {
    return props.children;
  });
  return (
    <KCheckbox.Root
      title={props.title}
      checked={!!props.state}
      onChange={props.setState}
      class={css({
        display: "flex",
        alignItems: "center",
        gap: "0.5rem",
        "& input:focus-visible + .control, & .control:hover": {
          boxShadow: "0 0 0 1px white, 0 0 0 3px var(--np-accent-400)",
        },
      })}
    >
      <KCheckbox.Input class="checkbox__input" />
      <KCheckbox.Control
        class={
          "control " +
          css({
            borderRadius: "0.25rem",
            borderWidth: "1px",
            boxShadow:
              "0 1px 3px 0 rgb(0 0 0 / 0.1), 0 1px 2px -1px rgb(0 0 0 / 0.1)",
            borderColor: props.state ? "#4b5563" : "#6b7280",
            backgroundColor: props.state ? "#6b7280" : "#9ca3af",
            color: props.state ? "white" : "#4b5563",
            boxSizing: "border-box",
            padding: "calc(0.25rem - 1px)",
            transition: "all 150ms cubic-bezier(0.4, 0, 0.2, 1)",
            lineHeight: 1,
            "&:hover": {
              cursor: "pointer",
            },
          })
        }
      >
        <Show when={!props.state}>
          <CheckIcon style={{ visibility: "hidden" }} />
        </Show>

        <KCheckbox.Indicator>
          <CheckIcon />
        </KCheckbox.Indicator>
      </KCheckbox.Control>
      <Show when={child_()}>
        <KCheckbox.Label>{child_()}</KCheckbox.Label>
      </Show>
    </KCheckbox.Root>
  );
};

export default Checkbox;
