import { Dialog as Kobalte } from "@kobalte/core";
import { css } from "../../css";
import {
  ComponentProps,
  FlowComponent,
  JSX,
  Show,
  ValidComponent,
  splitProps,
} from "solid-js";
import { CancelIcon, CheckIcon } from "./icons";
import { SmallButton } from "./buttons";
import { Dynamic } from "solid-js/web";

export const DialogButtons: FlowComponent = function (props) {
  return (
    <div
      class={css({
        display: "flex",
        alignItems: "center",
        justifyContent: "flex-end",
        marginTop: "0.75rem",
        gap: "0.5rem",
      })}
    >
      {props.children}
    </div>
  );
};

const Dialog: FlowComponent<
  { title: JSX.Element; showClose?: boolean } & ComponentProps<
    typeof Kobalte.Root
  >
> = function (props) {
  const [local, rootProps] = splitProps(
    props,
    ["children", "title", "showClose"],
    [
      "open",
      "defaultOpen",
      "onOpenChange",
      "modal",
      "preventScroll",
      "forceMount",
    ],
  );

  return (
    <Kobalte.Root {...rootProps}>
      <Kobalte.Portal>
        <Kobalte.Overlay
          class={css({
            position: "fixed",
            inset: 0,
            zIndex: 2100,
            backgroundColor: "#0003",
          })}
        />
        <div
          class={
            css({
              position: "fixed",
              inset: 0,
              zIndex: 2100,
              display: "flex",
              alignItems: "center",
              justifyContent: "center",
            }) + " np-admin-colors np-admin-font"
          }
        >
          <Kobalte.Content
            class={css({
              zIndex: 2100,
              maxWidth: "min(calc(100vw - 1rem),500px)",
              borderRadius: "0.5rem",
              padding: "1rem",
              backgroundColor: "#fff",
              boxShadow: "0 10px 15px -3px #0000001a,0 4px 6px -4px #0000001a",
            })}
          >
            <div
              class={css({
                display: "flex",
                alignItems: "baseline",
                justifyContent: "space-between",
                marginBottom: "0.75rem",
              })}
            >
              <Kobalte.Title
                class={css({
                  fontSize: "1.25rem",
                  fontWeight: 500,
                  color: "#334155",
                })}
              >
                {local.title}
              </Kobalte.Title>
              <Show when={local.showClose}>
                <Kobalte.CloseButton class="dialog__close-button">
                  <CancelIcon />
                </Kobalte.CloseButton>
              </Show>
            </div>
            {local.children}
          </Kobalte.Content>
        </div>
      </Kobalte.Portal>
    </Kobalte.Root>
  );
};

export const ConfirmDialog: FlowComponent<{
  open: boolean;
  icon?: ValidComponent;
  onConfirm: () => void;
  onCancel: () => void;
}> = function (props) {
  return (
    <Dialog
      open={props.open}
      onOpenChange={(open) => !open && props.onCancel()}
      title="Are you sure?"
    >
      <Show when={props.open}>
        <Kobalte.Description>{props.children}</Kobalte.Description>
        <DialogButtons>
          <SmallButton theme="primary" onClick={props.onConfirm}>
            <Dynamic component={props.icon || CheckIcon} /> Yes
          </SmallButton>
          <SmallButton onClick={props.onCancel}>
            <CancelIcon />
            Cancel
          </SmallButton>
        </DialogButtons>
      </Show>
    </Dialog>
  );
};

export default Dialog;
