import { Toast, toaster } from "@kobalte/core";
import { CancelIcon, CircleAlertIcon, CircleCheckIcon } from "./icons";
import classes from "./toast.module.css";
import { Show } from "solid-js";
import { css } from "../../css";

export const Region = function () {
  return (
    <Toast.Region pauseOnPageIdle={false} class="np-admin-colors np-admin-font">
      <Toast.List class={classes["toast__list"]} />
    </Toast.Region>
  );
};

export const toast = function (
  message: string,
  {
    status,
    description,
  }: { status?: "success" | "error"; description?: string } = {},
) {
  return toaster.show((props) => (
    <Toast.Root toastId={props.toastId} class={classes["toast"]}>
      <div class={classes["toast__content"]}>
        <div>
          <Toast.Title class={classes["toast__title"]}>
            <Show when={status === "success"}>
              <CircleCheckIcon class={css({ color: "var(--np-accent-500)" })} />
            </Show>
            <Show when={status === "error"}>
              <CircleAlertIcon class={css({ color: "#dc2626" })} />
            </Show>
            {message}
          </Toast.Title>
          <Show when={description}>
            <Toast.Description class={classes["toast__description"]}>
              {description}
            </Toast.Description>
          </Show>
        </div>
        <Toast.CloseButton class={classes["toast__close-button"]}>
          <CancelIcon />
        </Toast.CloseButton>
      </div>
      <Toast.ProgressTrack class={classes["toast__progress-track"]}>
        <Toast.ProgressFill class={classes["toast__progress-fill"]} />
      </Toast.ProgressTrack>
    </Toast.Root>
  ));
};
