import { createVisibilityObserver } from "@solid-primitives/intersection-observer";
import {
  Accessor,
  FlowComponent,
  createContext,
  createMemo,
  createSignal,
  useContext,
} from "solid-js";

const VisibilityObserverContext =
  createContext<ReturnType<typeof createSharedVisibilityObserver>>();

const createSharedVisibilityObserver = function () {
  const [mounted, setMounted] = createSignal(false);
  const useObserver = createVisibilityObserver(
    {
      rootMargin: "-50px -50px -50px -50px",
    },
    function (e) {
      setMounted(true);
      return e.isIntersecting;
    },
  );

  return [useObserver, mounted] as [typeof useObserver, typeof mounted];
};

export const VisibilityObserver: FlowComponent = function (props) {
  return (
    <VisibilityObserverContext.Provider
      value={createSharedVisibilityObserver()}
    >
      {props.children}
    </VisibilityObserverContext.Provider>
  );
};

export const useVisibilityObserver = function (
  el: Accessor<HTMLElement | undefined | false>,
) {
  const [useObserver, mounted] = useContext(VisibilityObserverContext)!;
  const visible = useObserver(el);
  return createMemo(function () {
    return mounted() && visible();
  });
};
