import { Component, Show } from "solid-js";
import { A, createRouteData, Outlet, Title, useParams } from "solid-start";
import { useAuth, A as ExternalA } from "nitropage";
import { css } from "nitropage/css";
import server$ from "solid-start/server";
import { prisma } from "../../../prisma";
import { FileIcon, SettingsIcon } from "../../common/icons";
import { authData } from "../../../server/auth";
import Container from "../container";
import { ProjectContext, useVisitLink } from "../../admin/util";
import { catchAuthErrors } from "../../../admin/auth";

const getProject = server$(async ({ projectId }: { projectId: string }) => {
  await authData(server$.request, { error: "throw" });

  return await prisma.nitroProject.findUniqueOrThrow({
    where: {
      id: projectId,
    },
  });
});

const Project: Component = function () {
  const params = useParams();
  const project = createRouteData(
    async (key) => {
      return await catchAuthErrors(() => getProject({ projectId: key[1] }));
    },
    {
      key: () => ["project", params.id],
    },
  );
  const visitLink = useVisitLink();

  return (
    <ProjectContext.Provider value={project}>
      <Show when={project()?.title}>
        <Title>{`Project - ${project()?.title}`}</Title>
      </Show>
      <div
        class={css({
          backgroundColor: "#f1f5f9",
        })}
      >
        <Container>
          <div
            class={css({
              padding: "1rem 0rem",
            })}
          >
            <div
              class={css({
                marginBottom: "0.5rem",
                display: "flex",
                gap: "1rem",
                alignItems: "baseline",
              })}
            >
              <h2
                class={css({
                  fontWeight: 500,
                  fontSize: "1.5rem",
                  letterSpacing: "0.05rem",
                })}
              >
                {project()?.title}
              </h2>
              <Show when={project()?.domain}>
                <ExternalA
                  class={css({
                    color: "#8b5cf6",
                    fontSize: "1.125rem",
                  })}
                  href={visitLink(`//${project()?.domain}`)}
                >
                  {project()?.domain}
                </ExternalA>
              </Show>
            </div>
            <div
              class={css({
                color: "#475569",
                display: "flex",
                gap: "1.75rem",
                "& a": {
                  display: "flex",
                  alignItems: "center",
                  gap: "0.2rem",
                },
                "& :hover": {
                  color: "#0f172a",
                },
                "& .active": {
                  color: "#0f172a",
                  fontWeight: 500,
                },
              })}
            >
              <A href={`/admin/project/${params.id}`} end activeClass="active">
                <FileIcon /> Pages
              </A>
              <A
                href={`/admin/project/${params.id}/settings`}
                end
                activeClass="active"
              >
                <SettingsIcon /> Settings
              </A>
            </div>
          </div>
        </Container>
      </div>

      <div
        class={css({
          paddingTop: "1rem",
        })}
      >
        <Outlet />
      </div>
    </ProjectContext.Provider>
  );
};

export default Project;
