import { Component, Show, useContext } from "solid-js";
import Container from "../container";
import {
  FormLayout,
  ProjectContext,
  useDomainValidator,
} from "../../admin/util";
import { Input } from "../../common/inputs";
import { ButtonSpinner, ConfirmButton } from "../../common/buttons";
import { SaveIcon } from "../../common/icons";
import { createRouteAction } from "solid-start";
import { updateProjectRpc } from "../../../server/rpc/project";
import { createForm, required } from "@modular-forms/solid";
import { useAuth } from "../../../hooks/auth";

type UpdateProjectForm = {
  title: string;
  domain: string;
};

const Settings: Component = function (props) {
  const project = useContext(ProjectContext)!;
  const [updateProjectState, updateProjectAction] = createRouteAction(
    async (params: Parameters<typeof updateProjectRpc>[0]) => {
      return await updateProjectRpc(params);
    },
  );
  const user = useAuth();
  const domainValidator = useDomainValidator(() => project()?.domain);
  const [projectForm, { Form, Field }] = createForm<UpdateProjectForm>();

  return (
    <Show when={project()}>
      {(project) => (
        <Form
          onSubmit={(data) =>
            updateProjectAction({
              csrf: user()?.csrf!,
              id: project().id,
              title: data.title,
              domain: data.domain,
            })
          }
        >
          <Container>
            <FormLayout>
              <Field
                name="title"
                validate={[required("Please specify your project title.")]}
              >
                {(field, props) => (
                  <Input
                    {...props}
                    error={field.error}
                    placeholder="Title"
                    label="Title"
                    theme="lg"
                    required
                    value={project().title}
                  />
                )}
              </Field>

              <Field name="domain" validate={[domainValidator]}>
                {(field, props) => (
                  <Input
                    {...props}
                    error={field.error}
                    placeholder="Domain"
                    label="Domain"
                    theme="lg"
                    value={project().domain!}
                  />
                )}
              </Field>

              <ConfirmButton type="submit">
                <ButtonSpinner show={updateProjectState.pending}>
                  <SaveIcon /> Save
                </ButtonSpinner>
              </ConfirmButton>
            </FormLayout>
          </Container>
        </Form>
      )}
    </Show>
  );
};

export default Settings;
