import { lazy } from "solid-js";
import { BlueprintData, Color, State } from "../../../types";
import type { RgbColor } from "vanilla-colorful/rgb-color-picker";
import { CssColor, css, rgb } from "../../../css";
import { rgbVal } from "../../page/colors";

export const cssFromRgbColor = function (color?: RgbColor, alpha = 1) {
  if (!color) {
    return;
  }

  const value = rgbVal(color);
  if (!value) return;

  return rgb({ value, alpha });
};

const id = "@npgjs/color";

export const colorClass = function (
  color?: CssColor,
  type: "color" | "bg" | "border" = "color",
) {
  if (!color?.value) return;
  let field: string = type;
  if (type === "bg") {
    field = "backgroundColor";
  }
  if (type === "border") {
    field = "borderColor";
  }

  return css({
    [field]: rgb(color),
  });
};

export type Value = Color | undefined;

export const getter = function (
  v: Color | undefined,
  state: State,
): CssColor | undefined {
  let value = v?.css;

  if (v?.id && state && state.colors[v.id]) {
    const invert = v.invert && state.colors[v.id].dark ? "-i" : "";
    value = `var(--${v.id}${invert})`;
  }

  if (!value) return;

  return {
    value,
    alpha: v?.alpha,
  };
};

const input = lazy(async () => import("./input"));
const colorData = (
  opts: { title?: BlueprintData["title"]; alpha?: boolean } = {},
) => {
  return {
    id,
    getter: function (v, s): CssColor | undefined {
      const result = getter(v, s);
      if (result && opts.alpha === false) {
        delete result.alpha;
      }

      return result;
    },
    defaultValue: () => undefined as Value,
    input,
    title: opts.title,
    custom: {
      alpha: opts.alpha,
    },
  } satisfies BlueprintData<Value>;
};

export default colorData;
