import { BlueprintDataInput, InputFromBlueprintData } from "../../../types";
import config from ".";
import { createId } from "@paralleldrive/cuid2";
import {
  BlueprintDataInputLayout,
  BlueprintDataTitle,
  BlueprintSettingsContainer,
} from "../../editor/sidebar/blueprintData";
import {
  createMemo,
  createSignal,
  For,
  Show,
  Suspense,
  useContext,
} from "solid-js";
import { css } from "nitropage/css";
import { Dynamic } from "solid-js/web";
import {
  ChevronDownIcon,
  ArrowsMoveIcon,
  PlusIcon,
  TrashIcon,
} from "../../common/icons";
import { createOnEnd, sortable } from "../../../directives/sortable";
import { SmallButton } from "../../common/buttons";
import { StateContext } from "../../context/page";

const Input: InputFromBlueprintData<ReturnType<typeof config>> = (p) => {
  sortable;
  const [state, setState] = useContext(StateContext)!;
  const Header = function () {
    return (
      <>
        <BlueprintDataTitle {...p}>
          <SmallButton
            theme="transparent"
            onClick={() => {
              const newId = createId();
              p.setValue(function (draft) {
                draft.byId[newId] = Object.entries(
                  p.data!.custom!.itemData,
                ).reduce(function (acc, [key, data]) {
                  acc[key] = data.defaultValue();
                  return acc;
                }, {} as any);
                draft.ids.push(newId);
                return draft;
              });
              p.setOverride(() => true);
              if (activeItem()) {
                setActiveItem(newId);
              }
            }}
          >
            <PlusIcon stroke-width={"4px"} />
          </SmallButton>
        </BlueprintDataTitle>
      </>
    );
  };
  const [activeItem, setActiveItem] = createSignal<undefined | string>();

  return (
    <BlueprintDataInputLayout header={<Header />}>
      <Show when={p.value?.ids.length}>
        <div
          class={css({
            padding: "0.5rem 1rem",
          })}
        >
          <div
            use:sortable={{
              handle: ".handle",
              animation: 150,
              forceFallback: true,
              onEnd: createOnEnd(function (childProduce) {
                p.setValue(function (draft) {
                  draft.ids = childProduce(draft.ids);
                  return draft;
                });
              }),
            }}
            class={css({
              display: "flex",
              flexDirection: "column",
              gap: "0.25rem",
            })}
          >
            <For each={p.value?.ids}>
              {(itemId, index) => {
                const item = createMemo(function () {
                  return p.value?.byId?.[itemId]!;
                });
                const title = createMemo(function () {
                  try {
                    return (
                      (p.data.custom?.itemTitle &&
                        p.data.custom?.itemTitle(item())) ||
                      itemId
                    );
                  } catch (err) {
                    console.error(err);
                  }

                  return itemId;
                });
                const active = createMemo(function () {
                  return activeItem() === itemId;
                });
                return (
                  <div
                    class={css({
                      borderRadius: "0.75rem",
                      backgroundColor: "#e2e8f0",
                      borderColor: "#e2e8f0",
                      borderWidth: "1px",
                    })}
                  >
                    <div
                      class={css({
                        borderRadius: "0.75rem",
                        backgroundColor: "white",
                        display: "flex",
                        gap: "1rem",
                        alignItems: "center",
                        paddingRight: "0.5rem",
                      })}
                    >
                      <button
                        class={css({
                          display: "flex",
                          gap: "0.25rem",
                          justifyContent: "space-between",
                          alignItems: "center",
                          overflow: "hidden",
                          flex: "1",
                          borderRadius: "0.75rem",
                          borderTopRightRadius: "0.75rem",
                          borderBottomRightRadius: "0.75rem",
                          fontSize: "0.75rem",
                          textAlign: "left",
                          padding: "0.175rem 0.75rem",
                          ":hover": {
                            "--tw-ring-offset-shadow":
                              "var(--tw-ring-inset) 0 0 0 var(--tw-ring-offset-width) var(--tw-ring-offset-color)",
                            "--tw-ring-shadow":
                              "var(--tw-ring-inset) 0 0 0 calc(2px + var(--tw-ring-offset-width)) var(--np-accent-400)",
                            boxShadow:
                              "var(--tw-ring-offset-shadow), var(--tw-ring-shadow), var(--tw-shadow, 0 0 #0000)",
                          },
                        })}
                        onClick={() =>
                          setActiveItem(
                            activeItem() !== itemId ? itemId : undefined,
                          )
                        }
                      >
                        <span
                          class={css({
                            fontSize: "0.75rem",
                            paddingRight: "0.25rem",
                            fontWeight: 600,
                          })}
                        >
                          {index() + 1}
                        </span>
                        <span
                          class={css({
                            flex: 1,
                            whiteSpace: "nowrap",
                            textOverflow: "ellipsis",
                            overflow: "hidden",
                            fontWeight: active() ? 600 : 400,
                            "& br": {
                              display: "none",
                            },
                            "& a": {
                              pointerEvents: "none",
                            },
                            "& *": {
                              textDecorationLine: "none",
                              fontWeight:
                                (active() ? 600 : 400) + " !important",
                              color: "#0f172a",
                            },

                            "& .block": {
                              display: "inline !important",
                            },
                          })}
                          innerHTML={title()}
                        />
                        <ChevronDownIcon
                          class={css({
                            flexShrink: 0,
                            transform: active() ? "rotate(180deg)" : undefined,
                          })}
                        />
                      </button>
                      <button class="handle">
                        <ArrowsMoveIcon color="#374151" />
                      </button>
                      <button
                        onClick={() => {
                          p.setValue((draft) => {
                            delete draft.byId[itemId];
                            draft.ids.splice(
                              draft.ids.findIndex((v) => v === itemId),
                              1,
                            );
                            return draft;
                          });
                        }}
                      >
                        <TrashIcon color="#374151" />
                      </button>
                    </div>
                    <Show when={active()}>
                      <div
                        class={css({
                          padding: "0.5rem 0.25rem 0.25rem",
                        })}
                      >
                        <BlueprintSettingsContainer>
                          <Suspense>
                            <For
                              each={Object.entries(p.data.custom?.itemData!)}
                            >
                              {([k, v]) => (
                                <Dynamic<BlueprintDataInput>
                                  key={k}
                                  data={v}
                                  context="project"
                                  setValue={
                                    ((childProduce: Function) => {
                                      p.setValue(function (draft) {
                                        if (draft.byId[itemId][k] == null) {
                                          draft.byId[itemId][k] =
                                            v.defaultValue();
                                        }
                                        draft.byId[itemId][k] = childProduce(
                                          draft.byId[itemId][k],
                                        );

                                        return draft;
                                      });
                                    }) as any
                                  }
                                  setOverride={((...args: any) => {}) as any}
                                  element={p.element}
                                  component={v.input}
                                  value={item()[k]}
                                  override={false}
                                />
                              )}
                            </For>
                          </Suspense>
                        </BlueprintSettingsContainer>
                      </div>
                    </Show>
                  </div>
                );
              }}
            </For>
          </div>
        </div>
      </Show>
    </BlueprintDataInputLayout>
  );
};

export default Input;
