import { lazy } from "solid-js";
import { BlueprintData } from "../../../types";

const id = "@npgjs/number";

export type Value = { content: number };

const numberData = <T extends boolean = false>({
  title,
  min = 0,
  max = 100,
  step = 1,
  defaultValue,
  optional,
  wide,
}: {
  title?: BlueprintData["title"];
  min?: number;
  max?: number;
  step?: number;
  defaultValue?: number;
  optional?: T;
  wide?: boolean;
} = {}) => {
  type R = T extends true ? number | undefined : number;
  if (!optional && defaultValue == null) {
    defaultValue = 0;
  }

  const getter = function (v?: Value): R {
    if (v?.content != null) return v?.content as any;
    if (optional) return undefined as any;
    return defaultValue as any;
  };

  return {
    id: id,
    title,
    getter,
    defaultValue: () => ({ content: defaultValue }) as Value,
    input: lazy(async () => import("./input")),
    custom: {
      min,
      max,
      step,
      wide,
    },
  } satisfies BlueprintData<Value>;
};

export default numberData;
