import { lazy } from "solid-js";
import { BlueprintData, ToolbarOption } from "../../../types";
import { spread } from "../../../util";
import { lookupRteToolbar } from "../../../util/elementData";

const id = "@npgjs/rte";

export const toolbarDefaults: ToolbarOption = {
  bold: true,
  clean: true,
  color: true,
  italic: true,
  link: true,
  script: true,
  underline: true,
};

export const toolbarFull: ToolbarOption = {
  ...toolbarDefaults,
  align: true,
  background: true,
  indent: true,
  listBullet: true,
  listOrdered: true,
};

export type Value = {
  content: string;
};

const rteData = (
  opts: {
    title?: BlueprintData["title"];
    defaultValue?: () => string;
    toolbar?: ToolbarOption;
    lorem?: boolean;
    shortLorem?: boolean;
  } = {},
) => {
  const defaultValue = function (): Value {
    let content = "";

    let value: string | undefined;
    if (opts.defaultValue) {
      value = opts.defaultValue();
    } else if (opts.lorem) {
      value =
        "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.";
    } else if (opts.shortLorem) {
      value = "Lorem Ipsum";
    }

    if (value) {
      content = `<span class="block">${value}</span>`;
    }
    return { content };
  };

  return spread(
    {
      id,
      getter: (v: Value) => {
        if (lookupRteToolbar) {
          return opts.toolbar as unknown as string;
        }
        if (!v) {
          return "";
        }
        return v.content;
      },
      defaultValue,
      input: lazy(async () => import("./input")),
      title: opts.title,
    },
    {
      title: opts.title,
      rte: {
        toolbar: opts.toolbar,
      },
    },
  ) satisfies BlueprintData;
};

export default rteData;
