import RteEditor from "../../rteEditor";
import {
  BlueprintDataInputLayout,
  BlueprintDataTitle,
  BlueprintSetting,
  BlueprintSettingsContainer,
} from "../../editor/sidebar/blueprintData";
import { BlueprintDataInput } from "../../../types";
import { css } from "../../../css/fela";
import colorData from "../color";
import { createSignal, For, Show, useContext } from "solid-js";
import { toolbarDefaults, Value } from ".";
import { range } from "froebel";
import { SmallButton } from "../../common/buttons";
import { SettingsIcon } from "../../common/icons";
import { colorPresets } from "../../page/richtext";
import { produce } from "solid-js/store";
import { StateContext } from "../../context/page";

const RteDataInput: BlueprintDataInput<Value> = (p) => {
  const colorData_ = colorData();
  const [showSettings, setShowSettings] = createSignal<boolean>();
  const [state, setState] = useContext(StateContext)!;

  const Header = function () {
    return (
      <BlueprintDataTitle {...p}>
        <SmallButton
          theme="transparent"
          active={showSettings()}
          onClick={() => setShowSettings(!showSettings())}
        >
          <SettingsIcon />
        </SmallButton>
      </BlueprintDataTitle>
    );
  };

  return (
    <BlueprintDataInputLayout header={<Header />}>
      <div
        class={css({
          padding: "0.5rem 1rem",
        })}
      >
        <RteEditor
          toolbar={p.data.rte?.toolbar || toolbarDefaults}
          value={p.value?.content || ""}
          setValue={(v) => {
            p.setValue((draft) => {
              draft.content = v;
              return draft;
            });
          }}
        />
        <Show when={showSettings()}>
          <div class={css({ paddingTop: "1rem" })}>
            <BlueprintSettingsContainer>
              <For each={range(1, colorPresets, 1)}>
                {(preset) => {
                  const key = "dyn" + preset;
                  return (
                    <BlueprintSetting
                      data={colorData_}
                      context="project"
                      key={"Color " + preset}
                      value={{ value: state.richtext.textColors?.[key] }}
                      setter={(childProduce) => {
                        setState(
                          produce(function (draft) {
                            draft.richtext.textColors[key] = childProduce({
                              value: draft.richtext.textColors[key],
                            }).value;

                            return draft;
                          }),
                        );
                      }}
                    />
                  );
                }}
              </For>
            </BlueprintSettingsContainer>
          </div>
        </Show>
      </div>
    </BlueprintDataInputLayout>
  );
};

export default RteDataInput;
