import { lazy } from "solid-js";
import { BlueprintData } from "../../../types";

const id = "@npgjs/select";

export type Value<T = any> = { selected: T };
export type Option<T extends string = string> = {
  title?: () => string;
  value?: T;
};
export type SelectOptions<V extends string> = Option<V>[];

const selectData = <
  V extends string,
  T extends SelectOptions<V> = SelectOptions<V>,
>({
  title,
  wide,
  options,
}: {
  title?: BlueprintData["title"];
  wide?: boolean;
  options: T;
}) => {
  return {
    id: id,
    title,
    getter: function (v?: Value<V>): V {
      return v?.selected || ("" as any);
    },
    defaultValue: () => ({ selected: "" }) as Value<V>,
    input: lazy(async () => import("./input")),
    custom: {
      wide,
      options,
    },
  } satisfies BlueprintData<Value<V>>;
};

export default selectData;
