import { BlueprintDataInput } from "../../../types";
import {
  BlueprintDataInputLayout,
  BlueprintDataTitle,
} from "../../editor/sidebar/blueprintData";
import { Value, Option } from ".";
import { css } from "nitropage/css";
import { Select } from "../../common/inputs";
import { Show } from "solid-js";
import { capitalize } from "froebel";

const SelectInput: BlueprintDataInput<Value> = function (props) {
  const TheInput = function () {
    return (
      <Select
        options={props.data.custom.options.map((v: Option) => {
          const value = v.value || "";
          const result = {
            value,
            label: v.title ? v.title() : capitalize(value),
          };

          return result;
        })}
        state={props.value?.selected || ""}
        setState={(v: any) =>
          props.setValue(function (draft) {
            draft.selected = v;
            return draft;
          })
        }
      />
    );
  };

  const Header = function () {
    return (
      <BlueprintDataTitle {...props}>
        <Show when={!props.data.custom.wide}>
          <TheInput />
        </Show>
      </BlueprintDataTitle>
    );
  };

  return (
    <BlueprintDataInputLayout header={<Header />}>
      <Show when={props.data.custom.wide}>
        <div
          class={css({
            fontSize: "0.9rem",
            padding: "0.5rem 1rem",
          })}
        >
          <TheInput />
        </div>
      </Show>
    </BlueprintDataInputLayout>
  );
};

export default SelectInput;
