import { BlueprintDataInput } from "../../../types";
import {
  BlueprintDataInputLayout,
  BlueprintDataTitle,
} from "../../editor/sidebar/blueprintData";
import { Value } from ".";
import { css } from "nitropage/css";
import { Input, TextArea } from "../../common/inputs";
import { Show, createMemo } from "solid-js";
import { slugifyUrl } from "../../../admin/slugify";
import { Dynamic } from "solid-js/web";

const TextInput: BlueprintDataInput<Value> = function (props) {
  const isWide = createMemo(
    () => props.data.custom.wide || props.data.custom.multiline,
  );
  const TheInput = function () {
    return (
      <Dynamic<typeof Input>
        component={props.data.custom.multiline ? (TextArea as any) : Input}
        placeholder={props.data.title ? props.data.title() : undefined}
        value={props.value?.content || ""}
        autocorrect="off"
        onBlur={(e) => {
          if (!props.data.custom.sanitizeSlug) {
            return;
          }

          const sanitizedValue = slugifyUrl(e.currentTarget.value);
          if (sanitizedValue == props.value?.content) {
            return;
          }

          props.setValue(function (draft) {
            draft.content = sanitizedValue;
            return draft;
          });
        }}
        onInput={(e) =>
          props.setValue(function (draft) {
            draft.content = e.currentTarget.value;
            return draft;
          })
        }
      />
    );
  };
  const Header = function () {
    return (
      <BlueprintDataTitle {...props}>
        <Show when={!isWide()}>
          <TheInput />
        </Show>
      </BlueprintDataTitle>
    );
  };
  return (
    <BlueprintDataInputLayout header={<Header />}>
      <Show when={isWide()}>
        <div
          class={css({
            padding: "0.5rem 1rem",
            "& textarea": {
              fontSize: "0.75rem",
              fontFamily:
                'ui-monospace, SFMono-Regular, Menlo, Monaco, Consolas, "Liberation Mono", "Courier New", monospace',
            },
          })}
        >
          <TheInput />
        </div>
      </Show>
    </BlueprintDataInputLayout>
  );
};

export default TextInput;
