import { useContext } from "solid-js";
import { EditorUiContext } from "../../context/admin";
import { ConfirmDialog } from "../../common/dialog";
import { TrashIcon } from "../../common/icons";
import { produce } from "solid-js/store";
import { useDeleteElement } from "../../../hooks/element";
import { ConfigContext, StateContext } from "../../context/page";

const DeleteElementDialog = function () {
  const [config] = useContext(ConfigContext)!;
  const [state] = useContext(StateContext)!;
  const [editorUi, setEditorUi] = useContext(EditorUiContext)!;
  const deleteElement = useDeleteElement();
  const element = () =>
    editorUi.deletingElementId
      ? state.elements[editorUi.deletingElementId]
      : undefined;
  const blueprintId = () => element()?.blueprint;
  const blueprint = () =>
    blueprintId() ? config.blueprints[blueprintId()!] : undefined;

  return (
    <ConfirmDialog
      open={!!editorUi.deletingElementId}
      icon={TrashIcon}
      onConfirm={() => {
        deleteElement(editorUi.deletingElementId!);
      }}
      onCancel={() =>
        setEditorUi(produce((d) => (d.deletingElementId = undefined)))
      }
    >
      Do you really wanna delete the selected "
      <strong>{blueprint()?.title()}</strong>" element?
    </ConfirmDialog>
  );
};

export default DeleteElementDialog;
