import { createRouteData } from "solid-start";
import { getFonts, getRevision, getSettings } from "../../server/rpc";
import { EditorUiState } from "../../types";
import { useParams, useRouteData } from "@solidjs/router";
import { adminRouteData } from "../admin/routeData";
import { getPresetsRpc } from "../../server/rpc/project";

export const createRevisionData = function (editorUi: EditorUiState) {
  const data = useRouteData<typeof adminRouteData>();
  const revision = createRouteData(
    async ([_, revisionId]) => {
      if (!revisionId) {
        return;
      }
      return await getRevision(revisionId);
    },
    {
      key: () => {
        return [
          "revision",
          editorUi.selectedRevisionId || data.editorPageRevision()?.id,
        ];
      },
    },
  );
  revision();
  return revision;
};

export const createSettingsProjectData = function () {
  const data = useRouteData<typeof adminRouteData>();
  const settingsProjectData = createRouteData(
    async ([_, __, projectId]) => {
      return await getSettings({ projectId, inherit: false });
    },
    {
      key: () => {
        return ["settings", "project", data.editorPage()?.projectId];
      },
    },
  );
  settingsProjectData();
  return settingsProjectData;
};

export const createSettingsPageData = function () {
  const params = useParams();
  const settingsPageData = createRouteData(
    async ([_, __, pageId]) => {
      return await getSettings({ pageId, inherit: false });
    },
    {
      key: () => {
        return ["settings", "page", params.id];
      },
    },
  );
  settingsPageData();
  return settingsPageData;
};

export const createFontData = function () {
  const data = useRouteData<typeof adminRouteData>();
  const fontData = createRouteData(
    async ([_, projectId]) => {
      if (!projectId) return [];
      return await getFonts({ projectId });
    },
    {
      key: () => {
        return ["fonts", data.editorPage()?.projectId];
      },
    },
  );
  fontData();
  return fontData;
};

export const createPresetsData = function () {
  const data = useRouteData<typeof adminRouteData>();
  const presetsData = createRouteData(
    async ([_, projectId]) => {
      if (!projectId) return [];
      return await getPresetsRpc({ projectId });
    },
    {
      key: () => {
        return ["presets", data.editorPage()?.projectId];
      },
    },
  );
  presetsData();
  return presetsData;
};
