import {
  Component,
  Show,
  createContext,
  splitProps,
  useContext,
} from "solid-js";
import { SetStoreFunction, createStore } from "solid-js/store";

// This is a quick workaround accordion implementation until the following issue is fixed:
// https://github.com/kobaltedev/kobalte/issues/236
// TODO: Replace with kobalte accordion when its issue is fixed

const AccordionContext = createContext<
  [
    get: {
      value: any;
    },
    set: SetStoreFunction<{
      value: any;
    }>,
  ]
>();
const AccordionItemContext = createContext<any>();

export const AccordionRoot: Component<any> = function (props) {
  const [local, rest] = splitProps(props, ["children", "defaultValue"]);
  const store = createStore({
    value: local.defaultValue ? local.defaultValue[0] : undefined,
  });

  return (
    <div {...rest}>
      <AccordionContext.Provider value={store}>
        {local.children}
      </AccordionContext.Provider>
    </div>
  );
};

export const AccordionHeader: Component<any> = function (props) {
  return <div {...props} />;
};

export const AccordionItem: Component<any> = function (props) {
  const [local, rest] = splitProps(props, ["value"]);
  const [state, setState] = useContext(AccordionContext)!;
  return (
    <AccordionItemContext.Provider value={local.value}>
      <div data-expanded={state.value === local.value || undefined} {...rest} />
    </AccordionItemContext.Provider>
  );
};

export const AccordionTrigger: Component<any> = function (props) {
  const value = useContext(AccordionItemContext)!;
  const [state, setState] = useContext(AccordionContext)!;
  return (
    <button
      data-expanded={state.value === value || undefined}
      onClick={() => setState("value", value)}
      {...props}
    />
  );
};

export const AccordionContent: Component<any> = function (props) {
  const value = useContext(AccordionItemContext)!;
  const [local, rest] = splitProps(props, ["children"]);
  const [state, setState] = useContext(AccordionContext)!;
  return (
    <Show when={state.value === value}>
      <div data-expanded={state.value === value || undefined} {...rest}>
        {local.children}
      </div>
    </Show>
  );
};
