import { createMemo, For, useContext } from "solid-js";
import { produce } from "solid-js/store";
import { createElementData } from "./blueprintData";
import {
  SmallButton,
  ConfirmButton,
  LargeSelectButton,
} from "../../common/buttons";
import { Element } from "../../../types";
import { css, cssMemo } from "../../../css/fela";
import { blueprintAllowed } from "../../../admin/element";
import { blueprintEntries } from "../../../admin/blueprint";
import { ConfigContext, StateContext } from "../../context/page";
import { EditorUiContext } from "../../context/admin";
import { TabBody } from "./util";

const ChooseTypeTab = function () {
  const [state, setState] = useContext(StateContext)!;
  const [editorUi, setEditorUi] = useContext(EditorUiContext)!;

  const [declaration] = useContext(ConfigContext)!;
  const css2 = cssMemo(() => {
    return {
      display: "grid",
      gridTemplateColumns: "repeat(2, 1fr)",
      gap: "0.5rem",
    };
  });
  const css3 = cssMemo(() => {
    return {
      marginTop: "0.5rem",
      display: "flex",
      gap: "0.75rem",
    };
  });

  const blueprints = createMemo(function () {
    const element = state.elements[editorUi.creatingElementId!];
    const parentElement = element.parentId
      ? state.elements[element.parentId]
      : undefined;

    const entries = blueprintEntries(declaration.blueprints).filter(function ([
      _,
      blueprint,
    ]) {
      return blueprintAllowed(
        declaration,
        blueprint.id,
        parentElement?.blueprint,
        element.parentSlot,
      );
    });

    return entries;
  });

  return (
    <div
      class={css({ height: "100%", display: "flex", flexDirection: "column" })}
    >
      <div
        class={css({
          fontSize: "1.25rem",
          textAlign: "center",
          lineHeight: "1.25rem",
          fontWeight: 600,
          "--tw-text-opacity": 1,
          color: "rgb(51 65 85 / var(--tw-text-opacity))",
          marginTop: "0.5rem",
          marginBottom: "1rem",
        })}
      >
        <TabBody>Choose a blueprint for the new element</TabBody>
      </div>
      <div
        class={css({
          flex: 1,
          overflowX: "clip",
          overflowY: "scroll",
          padding: "0.75rem 0",
        })}
      >
        <TabBody>
          <div class={css2()}>
            <For each={blueprints()}>
              {([type, blueprint]) => {
                return (
                  <LargeSelectButton
                    onClick={() => {
                      setState(
                        "elements",
                        editorUi.creatingElementId!,
                        produce(function (draft) {
                          draft.blueprint = type;
                          const defaultData = createElementData(
                            declaration,
                            type,
                          );
                          if (blueprint.previewData) {
                            const previewData = Object.entries(
                              blueprint.previewData(),
                            ).reduce(
                              function (acc, next) {
                                acc[next[0]] = {
                                  value: next[1],
                                  override: true,
                                };
                                return acc;
                              },
                              {} as Element["data"],
                            );
                            draft.data = {
                              ...defaultData,
                              ...previewData,
                            };
                          } else {
                            draft.data = defaultData;
                          }
                        }),
                      );
                    }}
                    active={
                      state.elements[editorUi.creatingElementId!].blueprint ===
                      type
                    }
                  >
                    {blueprint.title()}
                  </LargeSelectButton>
                );
              }}
            </For>
          </div>
        </TabBody>
      </div>
      <TabBody>
        <div class={css3()}>
          <ConfirmButton
            disabled={
              state.elements[editorUi.creatingElementId!]?.blueprint == "none"
            }
            onClick={() => {
              setState(
                produce((draft) => {
                  draft.elements[editorUi.creatingElementId!].data =
                    createElementData(
                      declaration,
                      state.elements[editorUi.creatingElementId!].blueprint,
                    );
                }),
              );
              setEditorUi(
                produce((draft) => {
                  draft.selectedElementId = draft.creatingElementId;
                  draft.creatingElementId = undefined;
                  draft.sidebarStatus = "idle";
                  draft.toolbarStatus = "creating";
                }),
              );
              setEditorUi("sidebarIdleTab", "elements");
            }}
          >
            Create
          </ConfirmButton>
          <SmallButton
            onClick={() => {
              setState(
                produce((draft) => {
                  const element = draft.elements[editorUi.creatingElementId!];
                  if (element.parentId) {
                    const slotIdx = state.elements[element.parentId].slots[
                      element.parentSlot
                    ].elements.findIndex((v) => v === element.id);
                    draft.elements[element.parentId].slots[
                      element.parentSlot
                    ].elements.splice(slotIdx, 1);
                  } else {
                    const slotIdx = state.root[
                      element.parentSlot as "default"
                    ].findIndex((v) => v === element.id);
                    draft.root[element.parentSlot as "default"].splice(
                      slotIdx,
                      1,
                    );
                  }

                  delete draft.elements[editorUi.creatingElementId!];
                }),
              );
              setEditorUi(
                produce((draft) => {
                  draft.creatingElementId = undefined;
                  draft.sidebarStatus = "idle";
                  draft.toolbarStatus = "creating";
                }),
              );
            }}
          >
            Cancel
          </SmallButton>
        </div>
      </TabBody>
    </div>
  );
};

export default ChooseTypeTab;
