import {
  BlueprintSetting,
  BlueprintSettingsContainer,
  ProjectBlueprintSettings,
} from "./blueprintData";
import { Fonts } from "./fonts";
import {
  SettingsAccordionItem,
  SettingsAccordion,
  MetaImageSetting,
  FaviconSetting,
} from "./util";
import colorData from "../../data/color";
import { Suspense } from "solid-js";
import { css } from "../../../css";
import { useSetting } from "../../../admin/editor";

const ProjectTab = function () {
  const colorData_ = colorData();
  const [setting, setSetting] = useSetting(
    "pageBackgroundColor",
    () => "projects",
  );

  return (
    <>
      <div
        class={css({
          marginBottom: "1rem",
        })}
      >
        <BlueprintSettingsContainer>
          <Suspense>
            <BlueprintSetting
              data={colorData_}
              context="project"
              key={"Page Background"}
              value={{ value: setting() }}
              setter={(childProduce) => {
                setSetting(function (draft) {
                  return childProduce({
                    value: draft,
                  }).value;
                });
              }}
            />
            <MetaImageSetting from="projects" />
            <FaviconSetting from="projects" />
          </Suspense>
        </BlueprintSettingsContainer>
      </div>
      <SettingsAccordion defaultValue={"blueprintSettings"}>
        <SettingsAccordionItem title="Fonts" value="fonts">
          <Fonts />
        </SettingsAccordionItem>
        <SettingsAccordionItem
          title="Blueprint Settings"
          value="blueprintSettings"
        >
          <ProjectBlueprintSettings />
        </SettingsAccordionItem>
      </SettingsAccordion>
    </>
  );
};

export default ProjectTab;
