import {
  Component,
  FlowComponent,
  JSX,
  JSXElement,
  ParentComponent,
} from "solid-js";
import { css } from "../../../css/fela";
import { ChevronDownIcon } from "../../common/icons";
import {
  AccordionContent,
  AccordionHeader,
  AccordionItem,
  AccordionRoot,
  AccordionTrigger,
} from "./accordionWorkaround";
import clsx from "clsx";
import { BlueprintSetting } from "./blueprintData";
import mediaData from "../../data/media";
import { useSetting } from "../../../admin/editor";

export const TabSubtitle: FlowComponent<{
  class?: string;
  rightColumn?: JSX.Element;
}> = function (p) {
  return (
    <div
      class={clsx(
        css({
          display: "flex",
          alignItems: "center",
          justifyContent: "space-between",
          gap: "1rem",
          marginBottom: "0.5rem",
        }),
        p.class,
      )}
    >
      <h2
        class={css({
          display: "flex",
          alignItems: "center",
          gap: "0.475rem",
          fontWeight: 600,
        })}
      >
        {p.children}
      </h2>
      {p.rightColumn}
    </div>
  );
};

export const SettingsAccordion: FlowComponent<{ defaultValue?: any }> =
  function (props) {
    return (
      <AccordionRoot
        collapsible={true}
        defaultValue={props.defaultValue ? [props.defaultValue] : undefined}
        class={css({
          display: "flex",
          flexDirection: "column",
        })}
      >
        {props.children}
      </AccordionRoot>
    );
  };

export const SettingsAccordionItem: FlowComponent<{
  title: JSXElement;
  value: any;
}> = function (props) {
  return (
    <AccordionItem
      class={css({
        "&[data-expanded]:not(:last-child)": {
          marginBottom: "1rem",
        },
      })}
      value={props.value}
    >
      <AccordionHeader>
        <AccordionTrigger
          class={css({
            color: "#475569",
            "&:not([data-expanded]) h2": {
              fontWeight: "500",
            },
            width: "100%",
            "& svg": {
              transition: "transform .3s cubic-bezier(.87,0,.13,1)",
            },
            ":hover": {
              color: "inherit",
            },
            "&[data-expanded]": {
              color: "inherit",
            },
            "&[data-expanded] svg": {
              transform: "rotate(180deg)",
            },
          })}
        >
          <TabSubtitle rightColumn={<ChevronDownIcon stroke-width={3} />}>
            {props.title}
          </TabSubtitle>
        </AccordionTrigger>
      </AccordionHeader>
      <AccordionContent
        class={css({
          overflow: "hidden",
          margin: "0 -3px",
          padding: "0 3px",
          // animation: "npAccordionHide 300ms cubic-bezier(0.87, 0, 0.13, 1)",
          "&[data-expanded]": {
            // animation: "npAccordionShow 300ms cubic-bezier(0.87, 0, 0.13, 1)",
          },
        })}
      >
        <div
          class={css({
            padding: "3px 0",
          })}
        >
          {props.children}
        </div>
      </AccordionContent>
    </AccordionItem>
  );
};

export const SidebarCard: ParentComponent = function (props) {
  return (
    <div
      class={css({
        display: "flex",
        flexDirection: "column",
        borderRadius: "0.35rem",
        backgroundColor: "rgb(248 250 252)",
        boxShadow: "0 1px 2px 0 rgb(0 0 0 / 0.05)",
      })}
    >
      {props.children}
    </div>
  );
};

export const TabBody: ParentComponent = function (props) {
  return (
    <div
      class={css({
        padding: " 0 0.75rem",
      })}
    >
      {props.children}
    </div>
  );
};

export const MetaImageSetting: Component<{ from: "pages" | "projects" }> =
  function (props) {
    const [setting, setSetting] = useSetting("metaImage", () => props.from);
    const data = mediaData({ type: "image", max: 1 });

    return (
      <BlueprintSetting
        data={data}
        context="project"
        key={"Meta Image"}
        value={{ value: setting() }}
        setter={(childProduce) => {
          setSetting(function (draft) {
            const value = childProduce({
              value: draft || data.defaultValue(),
            }).value;
            const id = value.mediaIds[0];
            if (id) {
              return value;
            }
          });
        }}
      />
    );
  };

export const FaviconSetting: Component<{ from: "pages" | "projects" }> =
  function (props) {
    const [setting, setSetting] = useSetting("favicon", () => props.from);
    const data = mediaData({ type: "image", max: 1 });

    return (
      <BlueprintSetting
        data={data}
        context="project"
        key={"Favicon"}
        value={{ value: setting() }}
        setter={(childProduce) => {
          setSetting(function (draft) {
            const value = childProduce({
              value: draft || data.defaultValue(),
            }).value;
            const id = value.mediaIds[0];
            if (id) {
              return value;
            }
          });
        }}
      />
    );
  };
