import {
  ParentComponent,
  JSX,
  splitProps,
  useContext,
  Component,
  createSignal,
  createMemo,
  Show,
} from "solid-js";
import { css, cssMemo } from "../../css/fela";
import {
  PlusIcon,
  CursorTextIcon,
  ClickIcon,
  ArrowsMoveIcon,
  TrashIcon,
  LockIcon,
  LockOpenIcon,
  SunIcon,
  MoonIcon,
} from "../common/icons";
import clsx from "clsx";
import { produce } from "solid-js/store";
import { EditorUiContext } from "../context/admin";

const ToolbarButton: ParentComponent<
  {
    active?: boolean;
    showToolbar: boolean;
  } & JSX.HTMLAttributes<HTMLButtonElement>
> = (p) => {
  const [editorUi] = useContext(EditorUiContext)!;

  const [local, others] = splitProps(p, ["active", "class"]);
  const style = cssMemo(() => {
    const activeRingStyle = {
      "--tw-ring-offset-shadow":
        "var(--tw-ring-inset) 0 0 0 var(--tw-ring-offset-width) var(--tw-ring-offset-color)",
      "--tw-ring-shadow":
        "var(--tw-ring-inset) 0 0 0 calc(2px + var(--tw-ring-offset-width)) var(--tw-ring-color)",
      boxShadow:
        "var(--tw-ring-offset-shadow), var(--tw-ring-shadow), var(--tw-shadow, 0 0 #0000)",
    };

    const selectedStyle = {
      "--tw-bg-opacity": 1,
      borderColor: "rgb(255 255 255 / 1)",
      backgroundColor: "#334155DD",
      color: "white",
    };

    return {
      ":hover .circle--selectable": {
        ...selectedStyle,
        ...activeRingStyle,
      },
      "& .circle": {
        display: "flex",
        alignItems: "center",
        margin: "0.65rem 0.25rem",
        height: "2.25rem",
        width: "2.25rem",
        fontSize: "1rem",
        fontWeight: 700,
        lineHeight: "1.75rem",
        opacity:
          p.showToolbar || (local.active && editorUi.toolbarStatus !== "locked")
            ? "1"
            : "0",
        "--tw-bg-opacity": 1,
        backgroundColor: "#f1f5f9B0",
        "--tw-shadow":
          "0 20px 25px -5px rgb(0 0 0 / 0.1), 0 8px 10px -6px rgb(0 0 0 / 0.1)",
        "--tw-shadow-colored":
          "0 20px 25px -5px var(--tw-shadow-color), 0 8px 10px -6px var(--tw-shadow-color)",
        boxShadow:
          "var(--tw-ring-offset-shadow, 0 0 #0000), var(--tw-ring-shadow, 0 0 #0000), var(--tw-shadow)",
        borderWidth: "4px",
        borderRadius: "9999px",
        borderColor: "rgb(255 255 255 / 0.7)",
        "--tw-ring-opacity": 1,
        "--tw-ring-color": "var(--np-accent-500)",
        "--tw-ring-offset-color": "#fff",
        transitionProperty: "all",
        transitionTimingFunction: "cubic-bezier(0, 0, 0.2, 1)",
        transitionDuration: "100ms",
        "--tw-translate-x": "0rem",
        transform:
          "translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))",
        "> div": {
          width: "100%",
          height: "100%",
          display: "flex",
          alignItems: "center",
          justifyContent: "center",
        },
        "&.circle--active": activeRingStyle,
        "&.circle--selected": selectedStyle,
        "&.circle--lock": {
          color: "rgb(0 0 0 / 0.6)",
          cursor: "not-allowed",
        },
      },
      "&.toolbar-hidden .circle--selected": {
        transitionDuration: "1000ms",
        "--tw-translate-x": "1.5rem",
      },
    };
  });
  return (
    <button
      class={style() + (!p.showToolbar ? " toolbar-hidden" : "")}
      {...others}
      disabled={editorUi.toolbarStatus === "locked"}
    >
      <div
        class={clsx(
          "circle",
          local.class,
          local.active &&
            p.showToolbar &&
            editorUi.toolbarStatus !== "locked" &&
            "circle--active",
          local.active &&
            editorUi.toolbarStatus !== "locked" &&
            "circle--selected",
          editorUi.toolbarStatus !== "locked" &&
            p.showToolbar &&
            "circle--selectable",
          editorUi.toolbarStatus === "locked" && "circle--lock",
        )}
      >
        <div>{p.children}</div>
      </div>
    </button>
  );
};

// The idea was to show the toolbar when the mouse is hovering the sidebar, but the toolbar is in the way when an element is being hovered

const Toolbar: Component<{ visible: boolean }> = function (props) {
  const [editorUi, setEditorUi] = useContext(EditorUiContext)!;
  const [lockToolbar, setLockToolbar] = createSignal(true);
  const [showToolbar, setShowToolbar] = createSignal(false);
  const toolbarVisible = createMemo(function () {
    return lockToolbar() || showToolbar();
  });
  let showToolbarTimeout: NodeJS.Timeout | undefined = undefined;
  let hideToolbarTimeout: NodeJS.Timeout | undefined = undefined;
  const style = cssMemo(() => {
    return {
      height: "100%",
      position: "relative",
      padding: "4rem 0",
      "& .toolbar.toolbar--show": {
        backdropFilter: "blur(8px)",
        "--tw-translate-x": props.visible ? "0px" : "-4rem",
        "--tw-shadow-color": "rgb(15 23 42 / 0.2)",
        backgroundColor: "rgb(255 255 255 / 0.2)",
        transitionDuration: "150ms",
      },
      "& .toolbar": {
        zIndex: 2000,
        display: "flex",
        flexDirection: "column",
        justifyContent: "center",
        gap: "0rem",
        minHeight: "70%",
        overflow: "visible",
        position: "absolute",
        top: "50%",
        left: "0px",
        "--tw-translate-y": "-50%",
        "--tw-translate-x": "-2.5rem",
        transform:
          "translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))",
        backgroundColor: "#FFF0",
        borderTopRightRadius: "1rem",
        borderBottomRightRadius: "1rem",
        transitionProperty: "all",
        transitionTimingFunction: "cubic-bezier(0.4, 0, 0.2, 1)",
        transitionDuration: "350ms",
        "--tw-shadow-color": "rgb(30 41 59 / 0)",
        "--tw-shadow": "var(--tw-shadow-colored)",
        "--tw-shadow-colored":
          "0 20px 25px -5px var(--tw-shadow-color), 0 8px 10px -6px var(--tw-shadow-color)",
        boxShadow:
          "var(--tw-ring-offset-shadow, 0 0 #0000), var(--tw-ring-shadow, 0 0 #0000), var(--tw-shadow)",
      },
    };
  });

  const toggleToolbar = function (status: typeof editorUi.toolbarStatus) {
    setEditorUi(
      "toolbarStatus",
      editorUi.toolbarStatus === status ? "idle" : status,
    );
  };

  return (
    <div
      class={style()}
      onMouseEnter={() => {
        if (!showToolbarTimeout) {
          showToolbarTimeout = setTimeout(function () {
            setShowToolbar(true);
          }, 50);
        }
        if (hideToolbarTimeout) {
          clearTimeout(hideToolbarTimeout);
          hideToolbarTimeout = undefined;
        }
      }}
      onMouseLeave={() => {
        if (showToolbarTimeout) {
          clearTimeout(showToolbarTimeout);
          showToolbarTimeout = undefined;
        }
        if (!hideToolbarTimeout) {
          hideToolbarTimeout = setTimeout(function () {
            setShowToolbar(false);
          }, 200);
        }
      }}
    >
      <div class={clsx("toolbar", toolbarVisible() && "toolbar--show")}>
        <div
          class={css({
            top: 0,
            position: "absolute",
          })}
        >
          <ToolbarButton
            title={lockToolbar() ? "Auto hide the toolbar" : "Lock the toolbar"}
            showToolbar={toolbarVisible()}
            onClick={() => {
              setLockToolbar(!lockToolbar());
            }}
          >
            <Show when={lockToolbar()} fallback={<LockOpenIcon />}>
              <LockIcon />
            </Show>
          </ToolbarButton>
        </div>
        <div
          class={css({
            bottom: 0,
            position: "absolute",
          })}
        >
          <ToolbarButton
            title={`Switch to ${
              editorUi.darkMode ? "light" : "dark"
            } color scheme`}
            showToolbar={toolbarVisible()}
            onClick={() => {
              setEditorUi(produce((d) => (d.darkMode = !d.darkMode)));
            }}
          >
            <Show when={editorUi.darkMode} fallback={<SunIcon />}>
              <MoonIcon />
            </Show>
          </ToolbarButton>
        </div>
        <ToolbarButton
          title="Select elements on the page"
          showToolbar={toolbarVisible()}
          active={editorUi.toolbarStatus === "selecting"}
          onClick={() => toggleToolbar("selecting")}
        >
          <ClickIcon />
        </ToolbarButton>
        <ToolbarButton
          title="Create new elements"
          showToolbar={toolbarVisible()}
          active={editorUi.toolbarStatus === "creating"}
          onClick={() => toggleToolbar("creating")}
        >
          <PlusIcon />
        </ToolbarButton>
        <ToolbarButton
          title="Write and format text"
          showToolbar={toolbarVisible()}
          active={editorUi.toolbarStatus === "write"}
          onClick={() => toggleToolbar("write")}
        >
          <CursorTextIcon />
        </ToolbarButton>
        <ToolbarButton
          title="Move elements"
          showToolbar={toolbarVisible()}
          active={editorUi.toolbarStatus === "moving"}
          onClick={() => toggleToolbar("moving")}
        >
          <ArrowsMoveIcon />
        </ToolbarButton>
        <ToolbarButton
          title="Eliminate elements"
          showToolbar={toolbarVisible()}
          active={editorUi.toolbarStatus === "deleting"}
          onClick={() => toggleToolbar("deleting")}
        >
          <TrashIcon />
        </ToolbarButton>
      </div>
    </div>
  );
};

export default Toolbar;
