import {
  Component,
  ComponentProps,
  createMemo,
  splitProps,
  useContext,
} from "solid-js";
import { A as SolidA, useLocation } from "solid-start";
import { Dynamic } from "solid-js/web";
import clsx from "clsx";
import { StateContext } from "../context/page";

const A: Component<
  ComponentProps<typeof SolidA> & {
    autoExternal?: boolean;
    matchHash?: boolean;
  }
> = function (props) {
  const path = useLocation();
  const [state] = useContext(StateContext) || [];
  const [_, restProps] = splitProps(props, [
    "children",
    "class",
    "end",
    "matchHash",
    "activeClass",
    "inactiveClass",
    "autoExternal",
  ]);
  const external = createMemo(function () {
    return (
      props.autoExternal !== false &&
      props.href.length &&
      (props.href.startsWith("//") ||
        (!props.href.startsWith("/") && !props.href.startsWith("#")))
    );
  });
  const urlMatch = createMemo(function () {
    if (state?.urlPath == null) return;
    const [href, hash = ""] = props.href.split("#");
    const hashMatch = props.matchHash === false || path.hash.slice(1) == hash;

    if (href == "") return hashMatch;
    if (props.end && state?.urlPath === href) return hashMatch;
    if (!props.end && href.startsWith(state?.urlPath)) return hashMatch;
  });
  return (
    <Dynamic
      component={state?.admin ? "span" : SolidA}
      class={clsx(
        props.class,
        urlMatch() ? props.activeClass : props.inactiveClass,
      )}
      target={external() ? "_blank" : undefined}
      rel={external() ? "noopener noreferrer" : undefined}
      {...restProps}
    >
      {props.children}
    </Dynamic>
  );
};

export default A;
