import { Component, createMemo, useContext } from "solid-js";
import { RgbColor } from "../../types";
import { Style } from "solid-start";
import { rgb } from "../../css";
import { getter } from "../data/color";
import { StateContext } from "../context/page";

export const rgbVal = function (color?: RgbColor) {
  if (!color) {
    return;
  }

  return `${color.r}, ${color.g}, ${color.b}`;
};

export const Colors: Component = function (p) {
  const [state] = useContext(StateContext)!;
  const colorCss = createMemo(function () {
    if (!state?.colors) {
      return;
    }

    const colorKeys = Object.keys(state.colors);
    if (!colorKeys.length) {
      return;
    }

    const lightEntries = [];
    const darkEntries = [];

    for (const key of colorKeys) {
      const light = state.colors[key].light;
      const dark = state.colors[key].dark;

      const lightRgb = rgbVal(light);
      lightEntries.push(`--${key}: ${lightRgb};`);

      if (dark) {
        const darkRgb = rgbVal(dark);
        lightEntries.push(`--${key}-i: ${darkRgb};`);
        darkEntries.push(`--${key}: ${darkRgb};`);
        darkEntries.push(`--${key}-i: ${lightRgb};`);
      }
    }

    return `
    :root {
      ${lightEntries.join("\n")}
    }
    
    .dark {
      ${darkEntries.join("\n")}
    }
`;
  });

  const backgroundColor = createMemo(function () {
    if (!state?.backgroundColor) {
      return;
    }

    const color = rgb(getter(state?.backgroundColor, state));
    if (!color) {
      return;
    }

    return `body { background-color: ${color}; }`;
  });

  const css = createMemo(function () {
    return (colorCss() || "") + (backgroundColor() || "");
  });

  return (
    <>
      <Style class={!state.admin ? "nitro-page-style" : undefined}>
        {css()}
      </Style>
    </>
  );
};
