import { Component, createMemo, useContext } from "solid-js";
import { getter } from "../data/color";
import { rgb } from "../../css";
import { Style } from "solid-start";
import { range } from "froebel";
import { StateContext } from "../context/page";

export const colorPresets = 13;

const Richtext: Component = function (p) {
  const [state] = useContext(StateContext)!;

  const renderGlobalColor = function (id: number) {
    const value = state.richtext.textColors["dyn" + id];
    if (!value) {
      return;
    }
    const color = rgb(getter(value, state));
    if (!color) {
      return;
    }
    return `
    .ql-color-g${id},
    .rte .ql-color-g${id} {
      color: ${color};
      --link-color: ${color};
      -webkit-text-fill-color: initial;
    }
    .ql-bg-g${id},
    .rte .ql-bg-g${id} {
      background-color: ${color} !important;
    }
    `;
  };
  const colorCss = createMemo(function () {
    if (!state.richtext.textColors) {
      return;
    }

    return range(1, colorPresets, 1).reduce(function (acc, item) {
      const css = renderGlobalColor(item);
      if (!css) {
        return acc;
      }
      acc += css;
      return acc;
    }, "");
  });

  return (
    <>
      <Style class={!state.admin ? "nitro-page-style" : undefined}>
        {colorCss()}
      </Style>
    </>
  );
};

export default Richtext;
