import { FlowComponent, createEffect, onCleanup, onMount } from "solid-js";
import { rehydrate, renderToMarkup } from "fela-dom";
import { isServer, useAssets } from "solid-js/web";
import { FelaContext, createFela } from "../../css/fela";
import "../../css/root.css";
import "../../css/root-rte.css";
import server$ from "solid-start/server";
import { renewAuth } from "nitropage/server";
import { createStore } from "solid-js/store";
import { Config } from "../../types";
import { useLogoutAction } from "../../hooks/auth";
import { isHydrated } from "@solid-primitives/lifecycle";
import { VisibilityObserver } from "../common/visibilityObserver";
import { ConfigContext } from "../context/page";

const renewSession = server$(async () => {
  return await renewAuth(server$.request, new Response());
});

export let hydrated = false;

const Root: FlowComponent<{ renewSessionInterval?: number }> = function (p) {
  const fela = createFela();
  const [_, logout] = useLogoutAction();

  const renew = function () {
    if (!document.cookie) {
      if (location.pathname.startsWith("/admin")) {
        logout();
      }
      return;
    }

    renewSession();
  };

  if (!isServer) {
    rehydrate(fela);
  }

  onMount(function () {
    const intervalSeconds = p.renewSessionInterval || 60 * 5;
    renew();
    const renewSessionInterval = setInterval(renew, intervalSeconds * 1000);
    onCleanup(function () {
      clearInterval(renewSessionInterval);
    });
  });

  createEffect(function () {
    hydrated = isHydrated();
  });

  useAssets(() => {
    return renderToMarkup(fela);
  });

  const configStore = createStore<Config>({ blueprints: {} });

  // TODO: This is an example how view transitions could be enabled in the future
  /*
  useBeforeLeave((e) => {
    if (document.startViewTransition) {
      e.preventDefault();
      document.startViewTransition(() => {
        e.retry(true);
      });
    }
  });
  */

  return (
    <VisibilityObserver>
      <FelaContext.Provider value={fela}>
        <ConfigContext.Provider value={configStore}>
          {p.children}
        </ConfigContext.Provider>
      </FelaContext.Provider>
    </VisibilityObserver>
  );
};

export default Root;
