import type { NitroPageRevision, NitroProject } from "@prisma/client";
import { formatISO } from "date-fns";
import { Component, For } from "solid-js";
import { getHttpHost } from "../../server/util";
import { NoHydration } from "solid-js/web";

declare module "solid-js" {
  namespace JSX {
    interface IntrinsicElements {
      urlset: any;
      url: any;
      loc: any;
      lastmod: any;
      changefreq: any;
    }
  }
}

const Sitemap: Component<{
  project: NitroProject;
  url: URL;
  revisions: NitroPageRevision[];
}> = function (props) {
  const host = getHttpHost(props.url, props.project.domain);

  return (
    <NoHydration>
      <urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">
        <For each={props.revisions}>
          {(page) => (
            <url>
              <loc>{host + page.urlPath}</loc>
              <lastmod>{formatISO(page.savedAt)}</lastmod>
              <changefreq>
                {page.urlPath === "/" ? "daily" : "weekly"}
              </changefreq>
            </url>
          )}
        </For>
      </urlset>
    </NoHydration>
  );
};

export default Sitemap;
