import { createContext, createMemo, useContext } from "solid-js";
import { createRenderer, IStyle } from "fela";
import prefixer from "fela-plugin-prefixer";
import multipleSelectors from "fela-plugin-multiple-selectors";

export type { IStyle } from "fela";

// specificityPrefix is needed because solid-start messes up the css file ordering in dev :)
export const createFela = () =>
  createRenderer({
    selectorPrefix: "f-",
    plugins: [prefixer(), multipleSelectors()],
    specificityPrefix: "html ",
  });
type Fela = ReturnType<typeof createFela>;
export const FelaContext = createContext<Fela>();
const id = (v: any) => v;
export const css = function (styles: IStyle) {
  const fela = useContext(FelaContext)!;
  return fela.renderRule(id, styles);
};
export const cssMemo = function (styles: () => IStyle) {
  return createMemo(function () {
    const nextStyles = styles();
    return css(nextStyles);
  });
};
