export { css, cssMemo } from "./fela";

// https://www.smashingmagazine.com/2022/01/modern-fluid-typography-css-clamp/
export const clamp = function (
  size: number,
  maxSize: number,
  { minScreen = 500, maxScreen = 1300, pxPerRem = 16, sizeUnit = "rem" } = {},
) {
  const inverse = maxSize < size;
  const size_ = !inverse ? size : maxSize;
  const maxSize_ = !inverse ? maxSize : size;
  const minScreen_ = !inverse ? minScreen : maxScreen;
  const maxScreen_ = !inverse ? maxScreen : minScreen;
  const sizePx = size_ * pxPerRem;
  const maxSizePx = maxSize_ * pxPerRem;

  const clampView = (100 * (maxSizePx - sizePx)) / (maxScreen_ - minScreen_);
  const clampPx =
    (minScreen_ * maxSizePx - maxScreen_ * sizePx) / (minScreen_ - maxScreen_);
  const clampRem = clampPx / pxPerRem;

  return `clamp(${size_}${sizeUnit}, ${clampView}vw + ${clampRem}${sizeUnit}, ${maxSize_}${sizeUnit})`;
};

export type CssColor = {
  value?: string;
  alpha?: number;
};

export const rgb = function (color?: CssColor) {
  if (!color?.value) return;
  const { value, alpha = 1 } = color;

  const fn = alpha === 1 ? "rgb" : "rgba";
  const appendAlpha = alpha !== 1 ? `, ${alpha}` : "";

  return `${fn}(${value}${appendAlpha})`;
};
