import { Resource } from "solid-js";
import { createRouteData, useRouteData } from "solid-start";
import { createRouteAction } from "solid-start/data/createRouteAction";
import server$ from "solid-start/server";
import { authData, clearAuth, signin, signup } from "nitropage/server";

export type AuthSession = {
  id: string;
  user: string;
  csrf: string;
};

export const createAuthData = <T extends () => Promise<any>>(fn: T) => {
  const resource = createRouteData(
    function (key, event) {
      return fn() as Awaited<ReturnType<T>>;
    },
    { key: () => "auth" },
  );
  // Start loading the auth data as soon as possible
  resource();
  return { auth: resource };
};

export const useAuth = (): Resource<Awaited<ReturnType<typeof authData>>> => {
  const data = useRouteData();
  return data?.auth;
};

const signup$ = server$(async function (args: {
  username: string;
  password: string;
  email: string;
}) {
  return await signup(new Response(), args);
});

export const useSignupAction = () => {
  return createRouteAction(
    async (args: Parameters<typeof signup$>[0]) => {
      const result = await signup$(args);
      return result;
    },
    { invalidate: ["auth"] },
  );
};

const signin$ = server$(async function (args: {
  username: string;
  password: string;
}) {
  return await signin(new Response(), args.username, args.password);
});

export const useSigninAction = () => {
  return createRouteAction(
    async (args: Parameters<typeof signin$>[0]) => {
      const result = await signin$(args);
      //await refetchRouteData("auth");
      return result;
    },
    { invalidate: ["auth"] },
  );
};

const logout$ = server$(async function () {
  return await clearAuth(new Response());
});

export const useLogoutAction = () => {
  return createRouteAction(
    async () => {
      await logout$();
    },
    { invalidate: ["auth"] },
  );
};
