import { useContext } from "solid-js";
import { produce } from "solid-js/store";
import { StateContext } from "../components/context/page";
import { EditorUiContext } from "../components/context/admin";

export const useDeleteElement = () => {
  const [state, setState] = useContext(StateContext)!;
  const [editorUi, setEditorUi] = useContext(EditorUiContext)!;

  const deleteElement = (elementId: string, clearParentSlots = true) => {
    const element = state.elements[elementId];
    const slot = element.parentSlot;
    const parentId = element.parentId;

    if (clearParentSlots) {
      if (!parentId) {
        setState(
          "root",
          slot as "default",
          produce((draft) => {
            draft.splice(draft.indexOf(element.id), 1);
          }),
        );
      } else {
        setState(
          "elements",
          produce((draft) => {
            draft[parentId].slots[slot].elements.splice(
              draft[parentId].slots[slot].elements.indexOf(element.id),
              1,
            );
          }),
        );
      }
    }

    for (const [_, slot] of Object.entries(element.slots)) {
      for (const slotElement of slot.elements) {
        deleteElement(slotElement, false);
      }
    }

    setEditorUi(
      produce((d) => {
        if (d.selectedElementId === elementId) {
          d.selectedElementId = undefined;
        }
        if (d.deletingElementId === elementId) {
          d.deletingElementId = undefined;
        }
      }),
    );

    // Without this some recursive element lists like breadcrumbs might ask for an element,
    // that already was deleted, giving them a frame to unmount first
    requestAnimationFrame(() => {
      setState(
        produce((d) => {
          delete d.elements[elementId];
        }),
      );
    });
  };

  return deleteElement;
};
