import { onMount } from "solid-js";
import { useNavigate } from "solid-start";
import { onClientCleanup } from "../util";

/**
 * Adds a global event handler for links in richtext elements. Internal links should use solid routers navigate method.
 */
export const useRteRouterLinks = function () {
  const navigate = useNavigate();
  const handler = function (event: MouseEvent) {
    const target = event.target as HTMLElement;
    if (!target) {
      return;
    }

    if (target.tagName != "A") {
      return;
    }

    if (target.getAttribute("rel")) {
      return;
    }

    if (!target.closest(".rte")) {
      return;
    }

    const href = target.getAttribute("href");
    if (!href) {
      return;
    }

    event.preventDefault();
    navigate(href);
  };

  onMount(function () {
    document.addEventListener("click", handler);
  });

  onClientCleanup(function () {
    document.removeEventListener("click", handler);
  });
};
