import { renderToStringAsync } from "solid-js/web";
import type { Middleware } from "solid-start/entry-server";
import Sitemap from "../components/page/sitemap";
import { getHttpHost, getProjectFromHostname } from "./util";
import { prisma } from "../prisma";
import { notFoundPage } from "./settings";
import { NitroProject } from "@prisma/client";
import { handleFeed } from "../components/page/feed";

const handleSitemap = async function ({
  url,
  project,
}: {
  url: URL;
  project: NitroProject;
}) {
  const notFoundPage_ = await notFoundPage({ projectId: project.id });

  const revisions = await prisma.nitroPageRevision.findMany({
    where: {
      page: {
        id: notFoundPage_.value ? { not: notFoundPage_.value } : undefined,
        projectId: project.id,
      },
      publishedPage: { id: { not: undefined } },
    },
  });

  const sitemap = await renderToStringAsync(() => (
    <Sitemap project={project} revisions={revisions} url={url} />
  ));

  return new Response('<?xml version="1.0" encoding="UTF-8"?>' + sitemap, {
    headers: {
      "Content-Type": "text/xml",
    },
  });
};

const handleRobots = async function ({
  url,
  project,
}: {
  url: URL;
  project: NitroProject;
}) {
  return new Response(`User-agent: *
Disallow:
  
Sitemap: ${getHttpHost(url, project.domain)}/sitemap.xml`);
};

export const createMiddleware = function ({} = {}): Middleware {
  return function ({ forward }) {
    return async function (event) {
      const url = new URL(event.request.url);

      const project = await getProjectFromHostname(url.hostname);
      if (!project) {
        return forward(event);
      }

      event.locals.nitroProject = project;
      event.locals.url = url;

      if (url.pathname === "/sitemap.xml") {
        return handleSitemap({ url, project });
      }

      if (url.pathname === "/feed" || url.pathname === "/feed/") {
        return handleFeed({ url, project });
      }

      if (url.pathname === "/robots.txt") {
        return handleRobots({ url, project });
      }

      return forward(event);
    };
  };
};
