import { prisma } from "../prisma";
import { flushSettingsCache } from "./settings/lib";

export const deletePage = async function (id: string) {
  const page = await prisma.nitroPage.findFirstOrThrow({
    include: {
      revisions: {
        include: { elements: true },
      },
    },
    where: {
      id,
    },
  });

  for (const revision of page.revisions) {
    await deletePageRevision(revision.id);
  }

  await prisma.nitroElement.deleteMany({
    where: {
      page: { id },
    },
  });

  const elementHistories = await prisma.nitroElementHistory.findMany({
    where: {
      page: { id },
    },
    include: {
      revisions: true,
    },
  });

  for (const history of elementHistories) {
    const elementIds = history.revisions.map((element) => element.id);
    await prisma.nitroElement.deleteMany({
      where: {
        id: {
          in: elementIds,
        },
      },
    });
    await prisma.nitroElementSlot.deleteMany({
      where: {
        parentElementId: {
          in: elementIds,
        },
      },
    });
  }

  await prisma.nitroElementHistory.deleteMany({
    where: {
      id: {
        in: elementHistories.map((history) => history.id),
      },
    },
  });

  await prisma.nitroPage.delete({
    where: {
      id: id,
    },
  });

  flushSettingsCache("pages", page.id);
};

export const deletePageRevision = async (id: string) => {
  const elements = await prisma.nitroElement.findMany({
    where: {
      page: { id },
    },
  });
  const elementIds = elements.map((el) => el.id);
  await prisma.nitroElementSlot.deleteMany({
    where: {
      OR: [
        { parentPageRevisionId: id },
        { parentElementId: { in: elementIds } },
      ],
    },
  });
  await prisma.nitroElement.deleteMany({
    where: {
      id: { in: elementIds },
    },
  });
  await prisma.nitroPageRevision.delete({
    where: {
      id: id,
    },
  });
};
