import { prisma } from "../prisma";
import { deleteMedia } from "./media";
import { deleteFont } from "./font";
import { deletePage } from "./page";
import { flushSettingsCache } from "./settings/lib";

export const deleteProject = async function ({ id }: { id: string }) {
  const project = await prisma.nitroProject.findUniqueOrThrow({
    include: {
      pages: true,
      fonts: true,
      media: true,
      settings: true,
    },
    where: {
      id,
    },
  });

  await prisma.nitroPreset.deleteMany({
    where: {
      projectId: project.id,
    },
  });

  await prisma.nitroSetting.deleteMany({
    where: {
      id: {
        in: project.settings.map((setting) => setting.id),
      },
    },
  });

  for (const media of project.media) {
    await deleteMedia(media.id);
  }

  for (const page of project.pages) {
    await deletePage(page.id);
  }

  for (const font of project.fonts) {
    await deleteFont(font.id);
  }

  await prisma.nitroProject.delete({
    where: {
      id,
    },
  });

  flushSettingsCache("projects", project.id);
};

export const updateProject = async function ({
  id,
  title,
  domain,
}: {
  id: string;
  title: string;
  domain: string;
}) {
  await prisma.nitroProject.update({
    data: {
      title,
      domain,
    },
    where: {
      id,
    },
  });
};

export const getDomainAvailability = async function ({
  domain,
}: {
  domain: string;
}) {
  const project = await prisma.nitroProject.findFirst({
    where: {
      domain,
    },
  });

  return {
    available: project == null,
  };
};
