import server$ from "solid-start/server";
import { authData } from "../auth";
import {
  deleteProject,
  getDomainAvailability,
  updateProject,
} from "../project";
import { pageEvent, throwOnDemo } from "../util";
import { prisma } from "../../prisma";

export const deleteProjectRpc = server$(
  async ({ csrf, id }: { csrf: string; id: string }) => {
    await authData(server$.request, { error: "throw", csrf: csrf || "" });
    throwOnDemo();
    await deleteProject({ id });
  },
);

export const updateProjectRpc = server$(
  async ({
    csrf,
    ...rest
  }: { csrf: string } & Parameters<typeof updateProject>[0]) => {
    await authData(server$.request, { error: "throw", csrf: csrf || "" });
    await updateProject(rest);
  },
);

export const getDomainAvailabilityRpc = server$(
  async ({ ...rest }: Parameters<typeof getDomainAvailability>[0]) => {
    await authData(server$.request, { error: "throw" });
    return await getDomainAvailability(rest);
  },
);

export const getPresetsRpc = server$(
  async ({ projectId }: { projectId: string }) => {
    const event = pageEvent(this);
    await authData(event.request, { error: "throw" });
    return await prisma.nitroPreset.findMany({
      where: {
        projectId,
      },
    });
  },
);
