import type {
  BlueprintDefaults,
  Color,
  Colors,
  RteTextColors,
} from "../../types";
import { combineSettings, createSetting } from "./lib";
import type { Value as MediaValue } from "../../components/data/media";

export const [preloadFonts, setPreloadFonts] = createSetting(
  "preloadFonts",
  async () => [] as string[],
  {},
);

export const [defaultFont, setDefaultFont] = createSetting(
  "defaultFont",
  async () => undefined as string | undefined,
  {},
);

export const [notFoundPage, setNotFoundPage] = createSetting(
  "notFoundPage",
  async () => undefined as string | undefined,
  {},
);

export const [metaDescription, setMetaDescription] = createSetting(
  "metaDescription",
  async () => undefined as string | undefined,
  {},
);

export const [metaImage, setMetaImage] = createSetting(
  "metaImage",
  async () => undefined as MediaValue | undefined,
  {},
);

export const [favicon, setFavicon] = createSetting(
  "favicon",
  async () => undefined as MediaValue | undefined,
  {},
);

export const [colors, setColors] = createSetting(
  "colors",
  async () => ({}) as Colors | undefined,
  {},
);

export const [pageBackgroundColor, setPageBackgroundColor] = createSetting(
  "pageBackgroundColor",
  async () => undefined as Color | undefined,
  {},
);

export const [rteTextColors, setRteTextColors] = createSetting(
  "rteTextColors",
  async () => ({}) as RteTextColors | undefined,
  {},
);

export const [blueprintDefaults, setBlueprintDefaults] = createSetting(
  "blueprintDefaults",
  async () => ({}) as BlueprintDefaults,
  {},
);

export const [getSettings, setSettings] = combineSettings({
  preloadFonts: [preloadFonts, setPreloadFonts],
  defaultFont: [defaultFont, setDefaultFont],
  notFoundPage: [notFoundPage, setNotFoundPage],
  metaDescription: [metaDescription, setMetaDescription],
  metaImage: [metaImage, setMetaImage],
  favicon: [favicon, setFavicon],
  colors: [colors, setColors],
  pageBackgroundColor: [pageBackgroundColor, setPageBackgroundColor],
  rteTextColors: [rteTextColors, setRteTextColors],
  blueprintDefaults: [blueprintDefaults, setBlueprintDefaults],
});

export type Settings = Awaited<ReturnType<typeof getSettings>>;
