import server$, { ServerError, useRequest } from "solid-start/server";
import type { CreateServerFunction } from "solid-start/server/server-functions/types";
import { demoEnabled } from "../util";
import { prisma } from "../prisma";

// Creates a higher order function accepting custom options for a server handler
// and properly redirects server$ (including server$.request) to the handler
export const withServerOptions = function <
  F extends (
    args: any,
    server: CreateServerFunction,
    options: any,
  ) => Promise<any>,
>(handler: F) {
  type O = Parameters<F>["2"];
  type A = Parameters<F>["0"];
  type R =
    Extract<A, undefined> extends never
      ? (this: CreateServerFunction, args: A) => ReturnType<F>
      : (this: CreateServerFunction, args?: A) => ReturnType<F>;
  type W =
    Extract<O, undefined> extends never
      ? (options: O) => R
      : (options?: O) => R;

  const wrapperFn = function (options: O) {
    return function (this: CreateServerFunction, args: A) {
      return handler(args, pageEvent(this), options);
    };
  };
  return wrapperFn as W;
};

export const withServer = function <
  F extends (args: any, server: CreateServerFunction) => Promise<any>,
>(handler: F) {
  type A = Parameters<F>["0"];
  type R =
    Extract<A, undefined> extends never
      ? (this: CreateServerFunction, args: A) => ReturnType<F>
      : (this: CreateServerFunction, args?: A) => ReturnType<F>;
  return function (this: CreateServerFunction, args: A) {
    return handler(args, pageEvent(this)) as ReturnType<F>;
  } as R;
};

export const pageEvent = function (thethis: any): CreateServerFunction {
  if (thethis.request) {
    return thethis;
  }

  const event = useRequest() as any;
  return event;
};

export const getRequest = function () {
  return server$.request || useRequest().request;
};

export const throwOnDemo = function () {
  if (!demoEnabled()) {
    return;
  }

  throw new ServerError("Unauthorized", { status: 401 });
};

export const getProjectFromHostname = async function (hostname: string) {
  let project = await prisma.nitroProject.findFirst({
    where: {
      domain: hostname,
    },
  });

  if (!project) {
    project = await prisma.nitroProject.findFirst({
      where: {
        OR: [{ domain: null }, { domain: "" }],
      },
    });
  }

  return project;
};

export const getHttpHost = function (url: URL, domain?: string | null) {
  const port = ["80", "443", ""].includes(url.port) ? "" : ":" + url.port;
  return `${url.protocol}//${domain || url.hostname}${port}`;
};
