import { createStore } from "solid-js/store";
import { State } from "../types";
import { createSignal, onMount } from "solid-js";

export const createAppStoreState = function (
  overrides: Partial<State> = {},
): State {
  return {
    cacheKey: 0,
    title: "",
    urlPath: "/",
    colors: {},
    richtext: {
      textColors: {},
    },
    fonts: [],
    root: {
      header: [],
      default: [],
      footer: [],
    },
    elements: {},
    presets: {},
    blueprintDefaults: {
      project: {},
    },
    ...overrides,
  };
};
export const createPageStore = function (
  pageId: string,
  overrides: Partial<State> = {},
) {
  // TODO: Finish the implementation of the local storage
  // Missing is the cleanup & recovery step of the storage
  /*
  const [state, setState, mounted] = createStorage<State>(
    createAppStoreState(),
    {
      timeout: 10000,
      prefix: `page-${pageId}`,
      storageApi: indexedDbStorage({ name: "nitropage-editor" }),
    }
  );
  */

  const [state, setState] = createStore(createAppStoreState(overrides));
  const [mounted, setMounted] = createSignal(false);

  onMount(function () {
    setTimeout(() => setMounted(true));
  });

  return [state, setState, mounted] as [
    typeof state,
    typeof setState,
    typeof mounted,
  ];
};
