import { useContext } from "solid-js";
import { isServer } from "solid-js/web";
import { Config, State } from "../types";
import { ConfigContext } from "../components/context/page";

const elementSettingsProxies: Record<string, { proxy: any }> = {};

export let lookupRteToolbar = false;
export const setLookupRteToolbar = function (value: boolean) {
  lookupRteToolbar = value;
};

export const getElementData = function (
  elementId: string,
  state: State,
  declaration: Config,
  property: string,
) {
  const element = state.elements[elementId];
  if (!element) {
    return;
  }

  const blueprint = declaration.blueprints[element.blueprint];
  const getter = blueprint.data[property]?.getter;

  if (element.data?.[property]?.override) {
    const value = element.data[property].value;
    return getter ? getter(value, state) : value;
  }

  const preset = element.presetId ? state.presets[element.presetId] : undefined;
  if (preset && preset.data?.[property]?.override) {
    const value = preset.data[property].value;
    return getter ? getter(value, state) : value;
  }

  const type = element.blueprint;
  if (!type) {
    return;
  }

  /*
  const pageSettings = state.blueprintDefaults.page[type];
  if (pageSettings && pageSettings[property]?.override) {
    // console.log('page', type, property, element.id, pageSettings[property].value)
    const value = pageSettings[property].value;
    return getter ? getter(value, state) : value;
  }
  */

  const projectSettings = state.blueprintDefaults.project[type];
  // console.log('global', type, property, element.id, globalSettings?.[property]?.value)
  const value = projectSettings?.[property]?.value;
  const defaultValue = blueprint.data[property]?.defaultValue;
  return getter ? getter(value != null ? value : defaultValue(), state) : value;
};

export const useElementData = function (state: State, elementId: string) {
  const [declaration] = useContext(ConfigContext)!;

  if (isServer) {
    const element = state.elements[elementId];
    if (!element) {
      return;
    }

    return Object.keys(element.data).reduce(function (acc, key) {
      acc[key] = getElementData(elementId, state, declaration, key);
      return acc;
    }, {} as any);
  }

  if (elementSettingsProxies[elementId] && typeof window !== "undefined") {
    return elementSettingsProxies[elementId].proxy;
  } else {
    elementSettingsProxies[elementId] = { memos: {} } as any;
  }
  const ctx = elementSettingsProxies[elementId];

  const proxy = new Proxy(
    {},
    {
      ownKeys() {
        const element = state.elements[elementId];
        return Object.keys(element.data);
      },
      getOwnPropertyDescriptor(target, prop) {
        return {
          enumerable: true,
          configurable: true,
        };
      },
      get(_, property: string) {
        return getElementData(elementId, state, declaration, property);
      },
      set(_, property, value) {
        return false;
      },
    },
  );

  elementSettingsProxies[elementId].proxy = proxy;
  return proxy;
};
