import { NitroMedia } from "@prisma/client";

export const imageTypes = [
  "image/jpeg",
  "image/png",
  "image/webp",
  "image/gif",
  "image/avif",
  "image/tiff",
  "image/svg+xml",
];

export const videoTypes = ["video/mp4", "video/ogg", "video/webm"];

export const audioTypes = ["audio/mpeg", "audio/wav", "audio/ogg"];

export const mediaType = function (media: NitroMedia) {
  return fileType(media.mime);
};

export const fileType = function (mime: string) {
  if (imageTypes.includes(mime)) {
    return "image";
  }

  if (audioTypes.includes(mime)) {
    return "audio";
  }

  if (videoTypes.includes(mime)) {
    return "video";
  }

  return "other";
};

const ratioCache: Record<string, number> = {};
export const calculateAspectRatio = function (aspectRatio?: string) {
  if (!aspectRatio) {
    return;
  }

  if (ratioCache[aspectRatio]) {
    return ratioCache[aspectRatio];
  }

  const parts = aspectRatio.split("/");
  if (parts.length !== 2) {
    return;
  }

  const result = Number.parseInt(parts[1]) / parseInt(parts[0]);
  ratioCache[aspectRatio] = result;

  return result;
};
