import path from "path";
import fs from "fs";
import ViteRestart from "./restartPlugin";
import { UserConfig } from "vite";

const resolveRealPath = (path_: string) =>
  fs.realpathSync(path.resolve(process.cwd(), path_));
const nitropageSrc = path.join(
  resolveRealPath("node_modules/nitropage/src"),
  "**",
);

/*
 * REMOVE node_modules AND REINSTALL AFTER CHANGING THESE
 */
const solidLibraries = [
  "@modular-forms/solid",
  "@kobalte/core",
  "@internationalized/message",
];
const serverLibraries = [
  "@hapi/iron",
  "@node-rs/xxhash",
  "bcrypt",
  "globby",
  "sharp",
  "zod",
  "fsevents",
];

const unique = function (input: any[]) {
  return [...new Set(input)];
};

const nitropage = function (config: UserConfig) {
  let isFlatNodeModules = false;
  try {
    resolveRealPath("node_modules/globby");
    isFlatNodeModules = true;
  } catch (err) {}

  config.resolve = config.resolve || {};
  config.resolve.dedupe = config.resolve.dedupe || [];
  // Without this, nitropage would import a duplicate copy of solid-start
  // TODO: Create a github issue in solid-start about this
  // Related: https://github.com/vitejs/vite/issues/8378
  config.resolve.dedupe.unshift("solid-start");

  config.optimizeDeps = config.optimizeDeps || {};

  // Optimize nitropage dependencies as early as possible
  config.optimizeDeps.entries = config.optimizeDeps.entries || [];
  if (!Array.isArray(config.optimizeDeps.entries)) {
    config.optimizeDeps.entries = [config.optimizeDeps.entries];
  }
  config.optimizeDeps.entries.unshift(nitropageSrc);

  config.optimizeDeps.exclude = config.optimizeDeps.exclude || [];
  config.optimizeDeps.exclude = unique([
    ...config.optimizeDeps.exclude,
    ...solidLibraries,
    ...serverLibraries,
  ]);

  // If the node_modules folder is flat we assume that this is a standalone nitropage project
  // Standalone nitropage projects need to exclude nitropage from optimizeDeps because of:
  // https://github.com/vitejs/vite/issues/5688
  // Optimally this should only exclude nitropage/server
  if (isFlatNodeModules) {
    config.optimizeDeps.exclude.unshift("nitropage");
  }

  config.optimizeDeps.include = config.optimizeDeps.include || [];
  config.optimizeDeps.include.unshift("isobject", "@lufrai/quill");

  config.ssr = config.ssr || {};
  config.ssr.external = config.ssr.external || [];
  config.ssr.external = unique([
    ...config.ssr.external,
    ...serverLibraries,
    ...[
      "vanilla-colorful/hex-alpha-color-picker.js",
      "@lufrai/quill",
      "isobject",
    ],
  ]);

  config.ssr.noExternal = config.ssr.noExternal || [];
  if (!Array.isArray(config.ssr.noExternal)) {
    config.ssr.noExternal = [config.ssr.noExternal as any];
  }
  config.ssr.noExternal = unique([...config.ssr.noExternal, ...solidLibraries]);

  /*
  console.log('optimizeDeps.include', config.optimizeDeps.include)
  console.log('optimizeDeps.exclude', config.optimizeDeps.exclude)
  console.log('ssr.external', config.ssr.external);
  console.log('ssr.noExternal', config.ssr.noExternal);
  */

  config.plugins = config.plugins || [];
  // TODO: Workaround for issue: https://github.com/vitejs/vite/issues/10616
  config.plugins.unshift(
    ViteRestart({
      restart: ["src/blueprints/**/*.np.{ts,tsx}"],
    }),
  );

  return config;
};

export default nitropage;
