import "solid-start/node/globals.js";
import manifest from "../../dist/public/route-manifest.json";
import entryHandler from "./entry-server.js";
import { join } from "path";
import { createRequest, handleNodeResponse } from "solid-start/node/fetch.js";

export const assetsPath = join(process.cwd(), "dist", "public");
export const publicPath = join(process.cwd(), "public");

const getStaticHTML = async (assetPath) => {
  let text = await readFile(join(assetsPath, assetPath + ".html"), "utf8");
  return new Response(text, {
    headers: {
      "content-type": "text/html",
    },
  });
};

export const handler = async function (request, response) {
  function internalFetch(route, init = {}) {
    if (route.startsWith("http")) {
      return fetch(route, init);
    }

    let url = new URL(route, "http://internal");
    const request = new Request(url.href, init);
    console.log("[internal]", url.method, url.href);
    return handler({
      request: request,
      clientAddress: req.socket.remoteAddress,
      locals: {},
      env,
      fetch: internalFetch,
    });
  }

  try {
    const res = createRequest(request);

    const webRes = await entryHandler({
      request: res,
      clientAddress: request.socket.remoteAddress,
      locals: {},
      env: {
        getStaticHTML,
        manifest,
      },
      fetch: internalFetch,
    });

    await handleNodeResponse(webRes, response);
  } catch (error) {
    console.error(error);
    res.statusCode = 500;
    res.statusMessage = "Internal Server Error";
    res.end();
  }
};
