import "dotenv/config";
import { handler, assetsPath, publicPath } from "./dist/server.js";
import polka from "polka";
import sirv from "sirv";

global.onunhandledrejection = (err, promise) => {
  console.error(err);
  console.error(promise);
};

const assetsHandler = sirv(assetsPath, {
  setHeaders: (res, pathname) => {
    if (!pathname.startsWith("/assets/")) {
      return;
    }

    res.setHeader("cache-control", "public, immutable, max-age=31536000");
  },
});

const publicHandler = sirv(publicPath, {
  dev: true,
});

const server = polka()
  .use("/", assetsHandler)
  .use("/", publicHandler)
  .use(handler);

const { PORT = 3000 } = process.env;

server.listen(PORT, (err) => {
  if (err) {
    console.log("error", err);
  } else {
    console.log(`Listening on port ${PORT}`);
  }
});
