import { A, createBlueprint, BlueprintComponent } from "nitropage";
import { css, colorData, rgb } from "nitropage/data/color";
import {
  Image,
  ImageContainer,
  mediaData,
  PlaceholderImage,
  useMedia,
  useResize,
} from "nitropage/data/media";
import { rteData } from "nitropage/data/rte";
import { textData } from "nitropage/data/text";
import { createResize } from "nitropage/server";
import { For, Show } from "solid-js";
import server$ from "solid-start/server";

const MyComponent: BlueprintComponent<typeof blueprint> = function (props) {
  const [authorImages] = useMedia(() => props.data.authorImage);
  const [optimizeMediaScreen] = useResize(
    server$(
      createResize({
        sizes: {
          author: { width: 100, height: 100 },
        },
      }),
    ),
  );

  return (
    <div
      class={
        "text-slate-900 " +
        css({
          color: rgb(props.data.textColor),
          "& .post-intro-text--light": {
            color: rgb({ ...props.data.textColor, alpha: 0.85 }),
          },
          "& .post-intro-text--lighter": {
            color: rgb({ ...props.data.textColor, alpha: 0.65 }),
          },
        })
      }
      {...props.elementProps}
    >
      {props.handles}
      <div class="flex items-center gap-4">
        <For
          each={
            authorImages().byOrder.length
              ? authorImages().byOrder
              : [PlaceholderImage]
          }
        >
          {(image) => {
            return (
              <ImageContainer
                width={false}
                height={false}
                class="overflow-hidden aspect-square shrink-0 w-14 h-14 rounded-full"
              >
                <Image
                  sources={optimizeMediaScreen(image, [{ size: "author" }])}
                  alt={image.description!}
                />
              </ImageContainer>
            );
          }}
        </For>

        <div class="flex flex-wrap items-center gap-3 lg:gap-6">
          <div class="w-full md:w-auto">
            <props.Rte data="author" component="span" class="text-base" />
            <props.Rte
              data="authorSubText"
              component="p"
              class="text-sm text-slate-500 post-intro-text--lighter"
            />
          </div>
          <Show when={props.data.category}>
            <div class="hidden md:block text-slate-700 post-intro-text--light">
              &bull;
            </div>
          </Show>
          <Show when={props.data.category || props.admin}>
            <A
              class="text-sm text-slate-700 post-intro-text--light hover:underline"
              href={props.data.categoryUrl}
            >
              <props.Rte data="category" />
            </A>
          </Show>

          <Show when={props.data.date}>
            <div class="text-slate-700 post-intro-text--light">&bull;</div>
          </Show>
          <props.Rte
            class="text-sm text-slate-700 post-intro-text--light"
            data="date"
          />
        </div>
      </div>
    </div>
  );
};

export const blueprint = createBlueprint(
  () => {
    return {
      title() {
        return "Blog Post - Intro";
      },
      acceptParentType: "element",
      slots: { default: {} },
      data: {
        category: rteData({
          title() {
            return "Category";
          },
          defaultValue() {
            return "Category";
          },
        }),
        categoryUrl: textData({
          title() {
            return "Category Url";
          },
          wide: true,
        }),
        date: rteData(),
        author: rteData({
          title() {
            return "Author";
          },
          defaultValue() {
            return "Author Name";
          },
        }),
        authorSubText: rteData({
          title() {
            return "Author Subtext";
          },
          defaultValue() {
            return "Author Subtext";
          },
        }),
        authorImage: mediaData({
          title() {
            return "Author Image";
          },
          type: "image",
          max: 1,
        }),
        textColor: colorData({
          title() {
            return "Text Color";
          },
          alpha: false,
        }),
      },
    };
  },
  import.meta.hot,
);

export default MyComponent;
export const id = "g5t3jgu1v9xwhb69c4wm52r5";
