import clsx from "clsx";
import { A, createBlueprint, BlueprintComponent } from "nitropage";
import { rgb, css } from "nitropage/data/color";
import { rteData } from "nitropage/data/rte";
import { selectData } from "nitropage/data/select";
import { textData } from "nitropage/data/text";
import { Show } from "solid-js";
import Button from "~/components/button";
import { buttonColorData, commonKeys } from "~/data/button";

const MyComponent: BlueprintComponent<typeof blueprint> = function (props) {
  return (
    <div
      class={clsx(
        "flex",
        props.data.align === "right" && "justify-end",
        props.data.align === "center" && "justify-center",
      )}
      {...props.elementProps}
    >
      <Button
        component={A}
        href={props.data.link}
        noActiveStyle
        clickScale={!props.admin}
        size={props.data.size as any}
        textColor={[rgb(props.data.textColor), rgb(props.data.textColorHover)]}
        bgColor={[rgb(props.data.bgColor), rgb(props.data.bgColorHover)]}
        class={clsx(
          css({
            transform: !props.lax ? "inherit !important" : undefined,
            "& .button__icon svg": {
              width: "1.3em",
              height: "1.3em",
            },
          }),
          props.data.align === "stretch" && "flex-1",
        )}
      >
        {props.handles}
        <Show when={props.data.iconSvg}>
          <span class="button__icon" innerHTML={props.data.iconSvg} />
        </Show>
        <props.Rte data="content" />
      </Button>
    </div>
  );
};

export const blueprint = createBlueprint(
  () => {
    return {
      acceptParentType: "element",
      data: {
        link: textData({
          wide: true,
        }),
        content: rteData({
          shortLorem: true,
          toolbar: {
            bold: true,
            underline: true,
            clean: true,
            color: true,
          },
        }),
        iconSvg: textData({
          wide: true,
        }),
        align: selectData<"" | "center" | "stretch" | "right">({
          options: [
            { title: () => "Left" },
            { value: "center" },
            { value: "stretch" },
            { value: "right" },
          ],
        }),
        size: selectData({
          options: [
            { title: () => "Small", value: "sm" },
            { title: () => "Medium" },
            { title: () => "Large", value: "lg" },
          ],
        }),
        ...buttonColorData({
          keys: commonKeys,
        }),
      },
    };
  },
  import.meta.hot,
);

export default MyComponent;
export const passive = true;
export const id = "m2vqbubuu61s8cu8l9z3cllu";
