import clsx from "clsx";
import { createBlueprint, BlueprintComponent, Richtext } from "nitropage";
import { colorData, css, rgb } from "nitropage/data/color";
import { fontCss, fontData } from "nitropage/data/font";
import { listData, sortItems } from "nitropage/data/list";
import { rteData } from "nitropage/data/rte";
import { textData } from "nitropage/data/text";
import { For, Show } from "solid-js";
import Heading from "~/components/heading";
import CatIcon from "~icons/tabler/cat";

/*
 * The design of this blueprint is inspired by Meraki UI, check it out!
 * https://github.com/merakiui/merakiui
 */

const MyComponent: BlueprintComponent<typeof blueprint> = function (props) {
  return (
    <div
      class={clsx(
        css({
          "& .contact__title": {
            ...fontCss(props.data.titleFont),
            color: rgb(props.data.titleColor),
          },
          "& .contact__title--faded": {
            color: rgb({ ...props.data.titleColor, alpha: 0.8 }),
          },
          "& .contact__highlight-text": {
            color: rgb(props.data.highlightColor),
          },
          "& .contact__icon": {
            backgroundColor: rgb(props.data.iconBackgroundColor),
          },
          "& .contact__icon svg": {
            width: "1.5rem",
            height: "1.5rem",
          },
          "& .contact__item": {
            backgroundColor: rgb(props.data.itemBackgroundColor),
            color: rgb(props.data.itemTextColor),
          },
          "& .contact__item-text": {
            color: rgb(props.data.itemTextColor),
          },
          "& .contact__item-text--faded": {
            color: rgb({ ...props.data.itemTextColor, alpha: 0.8 }),
          },
        }),
        "grid gap-8 lg:grid-cols-3",
      )}
      {...props.elementProps}
    >
      {props.handles}
      <div>
        <props.Rte
          component="p"
          data="subtitle"
          class="font-medium contact__highlight-text text-blue-600"
        />

        <Heading size="sm" level={2} class="contact__title mt-2">
          <props.Rte data="title" />
        </Heading>

        <props.Rte
          data="description"
          component="p"
          class="mt-3 contact__title--faded text-slate-500"
        />
      </div>

      <div class="grid grid-cols-1 gap-12 lg:col-span-2 sm:grid-cols-2 ">
        <For each={sortItems(props.data.items)}>
          {([item, id]) => (
            <div class="p-4 rounded-lg contact__item bg-blue-50 md:p-6">
              <span class="inline-block p-3 contact__icon contact__highlight-text text-blue-600 rounded-lg bg-blue-100/80">
                <Show when={item.iconSvg} fallback={<CatIcon />}>
                  <span innerHTML={item.iconSvg} />
                </Show>
              </span>

              <props.Rte
                path={(p) => p("items").byId[id].title}
                component="h3"
                class="mt-4 text-base font-medium contact__item-text text-slate-800"
              />
              <props.Rte
                path={(p) => p("items").byId[id].description}
                component="p"
                class="mt-2 text-sm contact__item-text--faded text-slate-600"
              />
              <props.Rte
                path={(p) => p("items").byId[id].contact}
                component="p"
                class="mt-2 text-sm text-blue-600 contact__highlight-text"
              />
            </div>
          )}
        </For>
      </div>
    </div>
  );
};

export const blueprint = createBlueprint(
  () => {
    return {
      acceptParentType: "element",
      title() {
        return "Contact Grid";
      },
      data: {
        title: rteData({ shortLorem: true }),
        subtitle: rteData({ shortLorem: true }),
        description: rteData({ lorem: true }),
        items: listData({
          defaultItems: 2,
          itemTitle(item) {
            return item.title?.content;
          },
          itemData: {
            title: rteData({
              shortLorem: true,
            }),
            description: rteData({
              lorem: true,
            }),
            contact: rteData({
              shortLorem: true,
            }),
            iconSvg: textData({
              wide: true,
            }),
          },
        }),
        titleFont: fontData(),
        iconBackgroundColor: colorData(),
        titleColor: colorData({ alpha: false }),
        itemTextColor: colorData({ alpha: false }),
        itemBackgroundColor: colorData(),
        highlightColor: colorData(),
      },
    };
  },
  import.meta.hot,
);

export default MyComponent;
export const passive = true;
export const id = "mk8emfe02k3ebqqwpf6dmsfg";
