import clsx from "clsx";
import { createBlueprint, BlueprintComponent } from "nitropage";
import { clamp, css } from "nitropage/css";
import {
  mediaData,
  MediaContainer,
  PlaceholderImage,
  useMedia,
  useResize,
} from "nitropage/data/media";
import { numberData } from "nitropage/data/number";
import { toggleData } from "nitropage/data/toggle";
import { createResize } from "nitropage/server";
import server$ from "solid-start/server";
import { Container } from "~/components/container";

const MyComponent: BlueprintComponent<typeof blueprint> = function (props) {
  const [media] = useMedia(() => props.data.image, { dataUrl: true });
  const [optimizeMedia, _, { Source }] = useResize(
    server$(
      createResize({
        aspectRatios: ["16/9"],
        sizes: {
          lg: { width: 1520 },
          md: { width: 1024 },
          sm: { width: 600 },
        },
      }),
    ),
  );
  const image = () => media().byOrder[0] || PlaceholderImage;

  return (
    <div {...props.elementProps}>
      <div
        class={clsx(
          "flex",
          props.data.inverseOnDesktop ? "lg:flex-row-reverse" : "lg:flex-row",
          props.data.slotFirstOnMobile ? "flex-col-reverse" : "flex-col",
        )}
      >
        {props.handles}
        <div class="flex-1 relative">
          <MediaContainer
            class="lg:!absolute"
            placeholder
            lazy
            fadeDuration={2000}
            media={image()}
            sources={optimizeMedia(image(), [
              { size: "lg", screen: 1024 },
              { size: "md", screen: 768, aspectRatio: "16/9" },
              { size: "sm", aspectRatio: "16/9" },
            ])}
          />
        </div>
        <div
          class={
            css({
              padding: `${clamp(
                props.data.slotFirstOnMobile ? 4 : 3,
                props.data.verticalPadding,
              )} 0 ${clamp(
                props.data.slotFirstOnMobile ? 3 : 4,
                props.data.verticalPadding,
              )} 0`,
            }) + " flex-1 flex justify-center items-center"
          }
        >
          <div class="lg:w-10/12">
            <Container>
              <props.Slot
                class="w-full flex flex-col gap-10 lg:gap-12"
                name="default"
              />
            </Container>
          </div>
        </div>
      </div>
    </div>
  );
};

export const blueprint = createBlueprint(
  () => {
    return {
      slots: { default: {} },
      data: {
        image: mediaData({
          type: "image",
          max: 1,
        }),
        verticalPadding: numberData({ step: 0.1, max: 20, defaultValue: 8 }),
        slotFirstOnMobile: toggleData(),
        inverseOnDesktop: toggleData(),
      },
    };
  },
  import.meta.hot,
);

export default MyComponent;
export const id = "why0uuxbciqwiujfv6w96o5z";
