import { createBlueprint, BlueprintComponent } from "nitropage";
import { css, colorData, rgb } from "nitropage/data/color";
import { fontCss, fontData } from "nitropage/data/font";
import { rteData } from "nitropage/data/rte";
import { textData } from "nitropage/data/text";
import { Show, createSignal } from "solid-js";
import Button from "~/components/button";
import Input from "~/components/form";
import Heading from "~/components/heading";
import Icon from "~icons/tabler/arrowBigRightLinesFilled";
import { id as footerId } from "./page/footer.np";
import { buttonColorData, prefixedKeys } from "~/data/button";
import clsx from "clsx";

const MyComponent: BlueprintComponent<typeof blueprint> = function (props) {
  const [showCaptcha, setShowCaptcha] = createSignal(false);

  return (
    <div {...props.elementProps}>
      {props.handles}
      <div class={"text-center"}>
        <Heading
          class={css({
            color: rgb(props.data.titleColor),
            ...fontCss(props.data.titleFont),
          })}
          level={2}
          size="sm"
          fullWidth={true}
        >
          <props.Rte data="title" />
        </Heading>
      </div>
      <form
        action={props.data.formUrl}
        method="post"
        onSubmit={
          props.admin
            ? (evt) => evt.preventDefault()
            : function (evt) {
                if (!props.data.captchaHtml) {
                  return;
                }

                if (showCaptcha()) {
                  return;
                }

                evt.preventDefault();
                setShowCaptcha(true);
              }
        }
      >
        <Show when={props.data.honeypotInputName}>
          <div class="absolute left-[-9999px]">
            <input
              name={props.data.honeypotInputName}
              tabindex="-1"
              autocomplete="off"
            />
          </div>
        </Show>
        <div class="flex flex-col justify-center gap-x-5 mx-auto mt-6 space-y-3 md:space-y-0 md:flex-row">
          <Input
            class="w-full md:w-auto"
            name={props.data.emailInputName || "email"}
            type="email"
            placeholder={props.data.emailInputPlaceholder || "Email address"}
            required
          />
          <Button
            clickScale={!props.admin}
            textColor={[
              rgb(props.data.buttonTextColor),
              rgb(props.data.buttonTextColorHover),
            ]}
            bgColor={[
              rgb(props.data.buttonBgColor),
              rgb(props.data.buttonBgColorHover),
            ]}
            type="submit"
            component={props.admin ? "a" : undefined}
          >
            <Icon /> <props.Rte data="buttonText" />
          </Button>
        </div>
        <Show when={props.data.captchaHtml}>
          <div
            class={clsx(
              "flex items-center justify-center mt-2",
              !showCaptcha() && "hidden",
            )}
            innerHTML={props.data.captchaHtml}
          />
        </Show>
        <Show when={props.data.hiddenHtml}>
          <div class="hidden" innerHTML={props.data.hiddenHtml} />
        </Show>
      </form>
    </div>
  );
};

export const blueprint = createBlueprint(
  () => {
    return {
      acceptParent: function (blueprint) {
        return blueprint === footerId;
      },
      title() {
        return "Footer Newsletter-Form";
      },
      data: {
        formUrl: textData({
          wide: true,
        }),
        emailInputName: textData(),
        emailInputPlaceholder: textData(),
        title: rteData({
          defaultValue() {
            return "Join the newsletter";
          },
        }),
        buttonText: rteData({
          defaultValue() {
            return "Subscribe";
          },
        }),
        ...buttonColorData({
          title: "Button",
          keys: prefixedKeys,
        }),
        titleColor: colorData(),
        titleFont: fontData({
          title() {
            return "Newsletter Title Font";
          },
          italic: true,
          letterSpacing: true,
          lineHeight: true,
          weight: true,
        }),
        hiddenHtml: textData({
          multiline: true,
        }),
        captchaHtml: textData({
          multiline: true,
        }),
        honeypotInputName: textData(),
      },
    };
  },
  import.meta.hot,
);

export default MyComponent;
export const id = "fmj4rug8jkbfy4lqe28n48wb";
