import clsx from "clsx";
import { createBlueprint, BlueprintComponent } from "nitropage";
import { css, colorData, rgb } from "nitropage/data/color";
import { fontData, fontCss } from "nitropage/data/font";
import { numberData } from "nitropage/data/number";
import { rteData } from "nitropage/data/rte";
import { selectData } from "nitropage/data/select";
import { toggleData } from "nitropage/data/toggle";
import { ToolbarOption } from "nitropage/src/types";
import Heading from "~/components/heading";

const MyComponent: BlueprintComponent<typeof blueprint> = function (props) {
  return (
    <div {...props.elementProps}>
      {props.handles}
      <Heading
        level={props.data.level}
        size={props.data.size}
        fullWidth={props.data.fullWidth}
        class={css({
          color: rgb(props.data.textColor),
          "& .heading__subtitle": {
            color: rgb(
              props.data.subtitleColor || {
                ...props.data.textColor,
                alpha: (props.data.textColor?.alpha ?? 1) * 0.65,
              },
            ),
            ...fontCss(props.data.subtitleFont),
          },
          ...fontCss(props.data.font),
        })}
        prependSubtitle={props.data.prependSubtitle}
        subtitleGap={props.data.subtitleGap}
        subtitle={<props.Rte data="subtitle" />}
      >
        <props.Rte data="content" />
      </Heading>
      <props.Slot
        class={clsx(
          ["", "sm", "md"].includes(props.data.size) && "mt-2 lg:mt-4",
          props.data.size === "lg" && "mt-4 lg:mt-6",
          props.data.size === "xl" && "mt-6 lg:mt-10",
        )}
        optional
        name="default"
      />
    </div>
  );
};

export const blueprint = createBlueprint(
  () => {
    const toolbar: ToolbarOption = {
      align: true,
      bold: true,
      italic: true,
      underline: true,
      clean: true,
      color: true,
      background: true,
    };

    const font = fontData({
      weight: true,
      italic: true,
      letterSpacing: true,
      responsive: true,
      size: true,
      family: true,
      lineHeight: true,
    });

    return {
      acceptParentType: "element",
      slots: {
        default: {
          max: 1,
        },
      },
      data: {
        content: rteData({
          title() {
            return "Text";
          },
          shortLorem: true,
          toolbar,
        }),
        subtitle: rteData({
          toolbar,
        }),
        level: numberData({
          title() {
            return "Heading Level";
          },
          defaultValue: 2,
          min: 1,
          max: 5,
        }),
        size: selectData<"" | "sm" | "md" | "lg" | "xl">({
          title() {
            return "Size Preset";
          },
          options: [
            { title: () => "Tiny" },
            { title: () => "Small", value: "sm" },
            { title: () => "Middle", value: "md" },
            { title: () => "Large", value: "lg" },
            { title: () => "Huge", value: "xl" },
          ],
        }),
        textColor: colorData(),
        subtitleColor: colorData(),
        fullWidth: toggleData({
          defaultValue: false,
        }),
        font,
        subtitleFont: font,
        subtitleGap: numberData({ optional: true, min: 0, max: 1, step: 0.1 }),
        prependSubtitle: toggleData(),
      },
    };
  },
  import.meta.hot,
);

export default MyComponent;
export const passive = true;
export const id = "zk7vp8rahquyz034aec4h2on";
