import clsx from "clsx";
import { createBlueprint, BlueprintComponent, A } from "nitropage";
import { colorClass, colorData } from "nitropage/data/color";
import { rteData, toolbarDefaults } from "nitropage/data/rte";
import { textData } from "nitropage/data/text";
import QRCode from "qrcode";
import { createMemo, createResource } from "solid-js";

const MyComponent: BlueprintComponent<typeof blueprint> = function (props) {
  const link = createMemo(() => props.data.link || "https://nitropage.com");
  const [qrcode] = createResource(link, (link) =>
    QRCode.toDataURL(link, {
      margin: 0,
    }),
  );
  return (
    <div
      class="flex gap-x-5 gap-y-2 flex-wrap break-inside-avoid"
      {...props.elementProps}
    >
      {props.handles}
      <A class="flex-shrink-0" href={link()}>
        <img
          src={qrcode()}
          class="bg-white rounded-md shadow hover:shadow-lg transition-all size-24 print:size-16 p-2 print:p-0 print:shadow-none print:rounded-none"
          alt="qrcode"
        />
      </A>
      <div class="min-w-44 flex-1">
        <props.Rte
          class={clsx(
            "text-sm text-slate-900 print:text-xs print:text-[0.625rem]",
            colorClass(props.data.textColor),
          )}
          data="description"
        />
        <div class="flex mt-1">
          <A
            class={clsx(
              "truncate text-blue-600 underline hover:no-underline text-xs print:text-xs print:break-all print:overflow-visible print:whitespace-normal",
              colorClass(props.data.linkColor),
            )}
            href={link()}
          >
            {link().replaceAll(/^https?:\/\//g, "")}
          </A>
        </div>
      </div>
    </div>
  );
};

export const blueprint = createBlueprint(
  () => {
    return {
      acceptParentType: "element",
      data: {
        link: textData({
          wide: true,
        }),
        description: rteData({
          shortLorem: true,
          toolbar: {
            ...toolbarDefaults,
            link: false,
          },
        }),
        textColor: colorData(),
        linkColor: colorData(),
      },
    };
  },
  import.meta.hot,
);

export default MyComponent;
export const passive = true;
export const id = "hxm8dhyn56up2pn9k2g31nng";
