import { createBlueprint, BlueprintComponent } from "nitropage";
import { css, colorData, rgb } from "nitropage/data/color";
import { listData, sortItems } from "nitropage/data/list";
import { rteData } from "nitropage/data/rte";
import { toggleData } from "nitropage/data/toggle";
import { For, Show } from "solid-js";
import CircleDashedIcon from "~icons/tabler/circle-dashed";
import CircleCheckIcon from "~icons/tabler/circle-check";
import { fontCss, fontData } from "nitropage/data/font";
import { numberData } from "nitropage/data/number";
import clsx from "clsx";

const MyComponent: BlueprintComponent<typeof blueprint> = function (props) {
  return (
    <div
      class={css({
        "& .roadmap__icon": {
          flexShrink: 0,
          color: rgb(props.data.iconColor),
        },
        "& .roadmap__icon.roadmap__icon--faded": {
          color: rgb({ ...props.data.iconColor, alpha: 0.8 }),
        },
        "& .roadmap__text": {
          color: rgb(props.data.textColor),
        },
        "& .roadmap__text--faded": {
          color: rgb({ ...props.data.textColor, alpha: 0.725 }),
        },
      })}
      {...props.elementProps}
    >
      {props.handles}
      <props.Rte
        class={
          css({ color: rgb({ ...props.data.textColor, alpha: 0.85 }) }) +
          " text-slate-700 block mb-8 text-lg lg:w-1/2"
        }
        data="description"
      />
      <div
        class={clsx(
          "grid gap-x-16 gap-y-9 flex-wrap",
          props.data.maxColumns >= 2 && "md:grid-cols-2",
          props.data.maxColumns == 3 && "xl:grid-cols-3",
          props.data.maxColumns == 4 && "xl:grid-cols-4",
        )}
      >
        <For each={sortItems(props.data.sections)}>
          {([g, sectionId]) => (
            <div>
              <props.Rte
                path={(d) => d("sections").byId[sectionId].title}
                class={
                  css({
                    ...fontCss(props.data.titleFont),
                  }) +
                  " roadmap__text text-slate-900 text-lg font-bold mb-4 block"
                }
              />
              <div class="flex flex-col gap-3">
                <For each={sortItems(g.items)}>
                  {([i, itemId]) => (
                    <div class="flex items-center gap-7">
                      <Show
                        when={i.checked}
                        fallback={
                          <CircleDashedIcon
                            class={
                              "roadmap__icon roadmap__icon--faded text-slate-700"
                            }
                          />
                        }
                      >
                        <CircleCheckIcon class="roadmap__icon" />
                      </Show>

                      <div class="flex flex-col">
                        <props.Rte
                          path={(d) =>
                            d("sections").byId[sectionId].items?.byId[itemId]
                              .title
                          }
                          class={"roadmap__text text-slate-950"}
                        />
                        <props.Rte
                          path={(d) =>
                            d("sections").byId[sectionId].items?.byId[itemId]
                              .description
                          }
                          class="roadmap__text--faded text-slate-600 text-sm"
                        />
                      </div>
                    </div>
                  )}
                </For>
              </div>
            </div>
          )}
        </For>
      </div>
    </div>
  );
};

export const blueprint = createBlueprint(
  () => {
    return {
      acceptParentType: "element",
      slots: { default: {} },
      data: {
        description: rteData({
          lorem: true,
        }),
        sections: listData({
          itemTitle(item) {
            return item.title?.content;
          },
          defaultItems: 2,
          itemData: {
            title: rteData({
              shortLorem: true,
            }),
            items: listData({
              defaultItems: 3,
              itemData: {
                checked: toggleData(),
                title: rteData({
                  shortLorem: true,
                }),
                description: rteData({
                  lorem: true,
                }),
              },
              itemTitle(item) {
                return item.title?.content;
              },
            }),
          },
        }),
        maxColumns: numberData({ min: 1, max: 4, defaultValue: 4 }),
        titleFont: fontData(),
        textColor: colorData({ alpha: false }),
        iconColor: colorData(),
      },
    };
  },
  import.meta.hot,
);

export default MyComponent;
export const passive = true;
export const id = "w6dnxkdsgi6xakmog46728lx";
