import { createBlueprint, BlueprintComponent } from "nitropage";
import PostLayout from "./blog/postLayout";
import clsx from "clsx";
import { toggleData } from "nitropage/data/toggle";
import Fragment from "~/components/fragment";
import { Dynamic, Show } from "solid-js/web";
import { colorData, css, rgb } from "nitropage/data/color";
import { selectData } from "nitropage/data/select";
import CircleIcon from "~icons/tabler/circleFilled";

const MyComponent: BlueprintComponent<typeof blueprint> = function (props) {
  return (
    <div
      class={clsx(
        !props.lax && "py-2",
        "text-slate-900",
        css({
          color: rgb(props.data.color),
        }),
      )}
      {...props.elementProps}
    >
      {props.handles}
      <Dynamic component={props.data.fullWidth ? Fragment : PostLayout}>
        <Show when={props.data.style === ""}>
          <div class="flex items-center justify-center gap-4 text-[0.25rem]">
            <CircleIcon />
            <CircleIcon />
            <CircleIcon />
          </div>
        </Show>
        <Show when={props.data.style !== ""}>
          <hr
            class={clsx(
              "border-current",
              props.data.style === "line-half" && "w-1/2 m-auto",
            )}
          />
        </Show>
      </Dynamic>
    </div>
  );
};

export const blueprint = createBlueprint(
  () => {
    return {
      acceptParentType: "element",
      data: {
        style: selectData<"" | "line" | "line-half">({
          options: [
            { title: () => "Dots" },
            { value: "line" },
            { value: "line-half" },
          ],
        }),
        color: colorData(),
        fullWidth: toggleData({ title: () => "Full Width" }),
      },
    };
  },
  import.meta.hot,
);

export default MyComponent;
export const passive = true;
export const id = "ybys9hh5umjkoi2kfkiidglr";
