import clsx from "clsx";
import { createBlueprint, BlueprintComponent, A } from "nitropage";
import { colorClass, colorData } from "nitropage/data/color";
import { rteData } from "nitropage/data/rte";
import { textData } from "nitropage/data/text";
import { toggleData } from "nitropage/data/toggle";
import { Dynamic } from "solid-js/web";

const MyComponent: BlueprintComponent<typeof blueprint> = function (props) {
  return (
    <div {...props.elementProps}>
      {props.handles}
      <Dynamic
        class={clsx(
          "inline-block rounded-full py-1 px-4 bg-slate-200 text-slate-800 text-xs lg:text-sm",
          props.data.uppercase && "uppercase",
          props.data.link && "hover:underline",
          colorClass(props.data.textColor),
          colorClass(props.data.bgColor, "bg"),
        )}
        component={props.data.link ? A : "div"}
        href={props.data.link}
      >
        <props.Rte data="content" />
      </Dynamic>
    </div>
  );
};

export const blueprint = createBlueprint(
  () => {
    return {
      acceptParentType: "element",
      data: {
        content: rteData({
          shortLorem: true,
          toolbar: {
            bold: true,
            clean: true,
            color: true,
          },
        }),
        link: textData({
          wide: true,
        }),
        bgColor: colorData(),
        textColor: colorData(),
        uppercase: toggleData(),
      },
    };
  },
  import.meta.hot,
);

export default MyComponent;
export const passive = true;
export const id = "ow7706ppsblfx9iyd3mbfd16";
