import { createBlueprint, BlueprintComponent } from "nitropage";
import { css, colorData, rgb } from "nitropage/data/color";
import { rteData, toolbarFull } from "nitropage/data/rte";
import { fontCss, fontData } from "nitropage/data/font";
import PostLayout from "./blog/postLayout";
import { toggleData } from "nitropage/data/toggle";
import { Dynamic } from "solid-js/web";
import Fragment from "~/components/fragment";
import clsx from "clsx";

const MyComponent: BlueprintComponent<typeof blueprint> = function (props) {
  return (
    <div {...props.elementProps}>
      {props.handles}
      <Dynamic component={props.data.fullWidth ? Fragment : PostLayout}>
        <props.Rte
          class={clsx(
            css({
              ...fontCss(props.data.textFont),
              color: rgb(props.data.textColor),
            }),
            "inline-block w-full text-slate-900 md:text-lg lg:text-xl",
            props.data.textFont.lineHeight == null && "xl:!leading-8",
          )}
          data="text"
        />
      </Dynamic>
    </div>
  );
};

export const blueprint = createBlueprint(
  () => {
    return {
      acceptParentType: "element",
      data: {
        text: rteData({
          title() {
            return "Text";
          },
          lorem: true,
          toolbar: toolbarFull,
        }),
        fullWidth: toggleData({ title: () => "Full Width" }),
        textColor: colorData({
          title() {
            return "Color";
          },
        }),
        textFont: fontData({
          title: () => "Font",
          weight: true,
          letterSpacing: true,
          lineHeight: true,
          italic: false,
          size: true,
        }),
      },
    };
  },
  import.meta.hot,
);

export default MyComponent;
export const passive = true;
export const id = "czbfkfg4rdgw5gf2543dv9r3";
