import clsx from "clsx";
import { createBlueprint, BlueprintComponent } from "nitropage";
import { css, colorData, rgb } from "nitropage/data/color";
import { selectData } from "nitropage/data/select";
import { createMemo } from "solid-js";

const MyComponent: BlueprintComponent<typeof blueprint> = function (props) {
  const style = createMemo(function () {
    return css({
      color: rgb(props.data.textColor),
      backgroundColor: rgb(props.data.bgColor),
    });
  });

  return (
    <div
      class={clsx(
        "flex flex-col",
        style(),
        props.data.padding === "" && "p-3 md:p-5 lg:p-6",
        props.data.padding === "sm" && "p-5 md:p-7 lg:p-9",
        props.data.padding === "md" && "p-7 md:p-12 lg:p-16",
        props.data.padding === "lg" && "p-9 md:p-20 lg:p-28",
        props.data.borderRadius === "sm" && "rounded-md",
        props.data.borderRadius === "md" && "rounded-xl",
        props.data.borderRadius === "lg" && "rounded-3xl",
        props.data.shadow === "sm" && "shadow-md",
        props.data.shadow === "md" && "shadow-xl",
        props.data.shadow === "lg" && "shadow-2xl",
      )}
      {...props.elementProps}
    >
      {props.handles}
      <props.Slot
        class="flex flex-1 flex-col gap-3 justify-between"
        name="default"
      />
    </div>
  );
};

export const blueprint = createBlueprint(
  () => {
    return {
      acceptParentType: "element",
      slots: { default: {} },
      data: {
        padding: selectData<"" | "sm" | "md" | "lg">({
          title() {
            return "Padding";
          },
          options: [
            {
              title() {
                return "Default";
              },
            },
            {
              title() {
                return "Small";
              },
              value: "sm",
            },
            {
              title() {
                return "Middle";
              },
              value: "md",
            },
            {
              title() {
                return "Large";
              },
              value: "lg",
            },
          ],
        }),
        borderRadius: selectData<"" | "sm" | "md" | "lg">({
          title() {
            return "Border Radius";
          },
          options: [
            {
              title() {
                return "None";
              },
            },
            {
              title() {
                return "Small";
              },
              value: "sm",
            },
            {
              title() {
                return "Middle";
              },
              value: "md",
            },
            {
              title() {
                return "Large";
              },
              value: "lg",
            },
          ],
        }),
        shadow: selectData<"" | "sm" | "md" | "lg">({
          title() {
            return "Shadow";
          },
          options: [
            {
              title() {
                return "None";
              },
            },
            {
              title() {
                return "Small";
              },
              value: "sm",
            },
            {
              title() {
                return "Middle";
              },
              value: "md",
            },
            {
              title() {
                return "Large";
              },
              value: "lg",
            },
          ],
        }),
        textColor: colorData({
          title() {
            return "Text Color";
          },
        }),
        bgColor: colorData({
          title() {
            return "Background-Color";
          },
        }),
      },
    };
  },
  import.meta.hot,
);

export default MyComponent;
export const id = "g05w6l6tg4qpkaqrir8dnwrj";
