import clsx from "clsx";
import { createBlueprint, BlueprintComponent } from "nitropage";
import { css } from "nitropage/data/color";
import { numberData } from "nitropage/data/number";
import { selectData } from "nitropage/data/select";
import { Component, ComponentProps, createMemo, Show } from "solid-js";

const MyComponent: BlueprintComponent<typeof blueprint> = function (props) {
  const Column: Component<ComponentProps<(typeof props)["Slot"]>> = function (
    p,
  ) {
    const style = createMemo(function () {
      return css({
        "&> *": {
          flex: props.data.justifyColumns === "stretch" ? 1 : undefined,
        },
      });
    });

    return (
      <props.Slot
        class={clsx(
          "flex flex-col gap-1",
          props.data.justifyColumns === "distribute" && "justify-between",
          props.data.justifyColumns === "stretch" && "justify-stretch",
          props.data.justifyColumns === "bottom" && "justify-end",
          style(),
        )}
        name={p.name}
        optional
      />
    );
  };

  return (
    <div {...props.elementProps}>
      {props.handles}
      <div
        class={clsx(
          "grid grid-cols-1 gap-8 gap-y-10 items-stretch",
          !props.lax && "py-4",
          props.data.quantity === 2 && "md:grid-cols-2",
          props.data.quantity === 3 && "lg:grid-cols-3",
          props.data.quantity === 4 && "md:grid-cols-2 lg:grid-cols-4",
        )}
      >
        <Column name="column1" />
        <Column name="column2" />

        <Show when={props.data.quantity >= 3}>
          <Column name="column3" />
        </Show>

        <Show when={props.data.quantity >= 4}>
          <Column name="column4" />
        </Show>
      </div>
    </div>
  );
};

export const blueprint = createBlueprint(
  () => {
    return {
      slots: { column1: {}, column2: {}, column3: {}, column4: {} },
      data: {
        quantity: numberData({
          title() {
            return "Columns";
          },
          defaultValue: 2,
          min: 2,
          max: 4,
        }),
        justifyColumns: selectData({
          options: [
            {
              title() {
                return "Top";
              },
            },
            {
              title() {
                return "Distribute";
              },
              value: "distribute",
            },
            {
              title() {
                return "Stretch";
              },
              value: "stretch",
            },
            {
              title() {
                return "Bottom";
              },
              value: "bottom",
            },
          ],
        }),
      },
    };
  },
  import.meta.hot,
);

export default MyComponent;
export const passive = true;
export const id = "hfd1h2d6cg67y3c4xvuyowow";
