import clsx from "clsx";
import { createBlueprint, BlueprintComponent } from "nitropage";
import { colorData, colorClass } from "nitropage/data/color";
import { selectData } from "nitropage/data/select";
import { Container } from "~/components/container";

const MyComponent: BlueprintComponent<typeof blueprint> = function (props) {
  return (
    <Container
      class={colorClass(props.data.backgroundColor, "bg")}
      {...props.elementProps}
    >
      <div class={clsx(!props.lax && "py-4")}>
        {props.handles}
        <props.Slot
          editorPadding={!props.data.topPadding && !props.data.bottomPadding}
          class={clsx(
            "flex flex-col gap-10 lg:gap-12",
            props.data.topPadding === "xs" && "pt-4 lg:pt-6 xl:pt-8",
            props.data.bottomPadding === "xs" && "pb-4 lg:pb-6 xl:pb-8",
            props.data.topPadding === "sm" && "pt-8 lg:pt-12 xl:pt-16",
            props.data.bottomPadding === "sm" && "pb-8 lg:pb-12 xl:pb-16",
            props.data.topPadding === "md" && "pt-16 lg:pt-20 xl:pt-24",
            props.data.bottomPadding === "md" && "pb-16 lg:pb-20 xl:pb-24",
            props.data.topPadding === "lg" && "pt-24 lg:pt-32 xl:pt-40",
            props.data.bottomPadding === "lg" && "pb-24 lg:pb-32 xl:pb-40",
            props.data.topPadding === "xl" && "pt-40 lg:pt-52 xl:pt-64",
            props.data.bottomPadding === "xl" && "pb-40 lg:pb-52 xl:pb-64",
            props.data.horizontalPadding === "sm" && "lg:px-8",
            props.data.horizontalPadding === "md" && "px-3 lg:px-12 xl:px-20",
            props.data.horizontalPadding === "lg" && "px-8 lg:px-20 xl:px-36",
            props.data.horizontalPadding === "xl" && "px-12 lg:px-32 xl:px-56",
          )}
          name="default"
        />
      </div>
    </Container>
  );
};

export const blueprint = createBlueprint(
  () => {
    const paddingOptions = [
      {
        title() {
          return "Small";
        },
        value: "sm",
      },
      {
        title() {
          return "Middle";
        },
        value: "md",
      },
      {
        title() {
          return "Large";
        },
        value: "lg",
      },
      {
        title() {
          return "Huge";
        },
        value: "xl",
      },
    ];
    const none = {
      title() {
        return "None";
      },
    };
    const horizontalPadding = [none, ...paddingOptions];
    const verticalPaddingOptions = [
      none,
      { value: "xs", title: () => "Tiny" },
      ...paddingOptions,
    ];

    return {
      slots: { default: {} },
      data: {
        backgroundColor: colorData({
          title() {
            return "Background Color";
          },
        }),
        topPadding: selectData<"" | "xs" | "sm" | "md" | "lg" | "xl">({
          title() {
            return "Top Padding";
          },
          options: verticalPaddingOptions as any,
        }),
        bottomPadding: selectData<"" | "xs" | "sm" | "md" | "lg" | "xl">({
          title() {
            return "Bottom Padding";
          },
          options: verticalPaddingOptions as any,
        }),
        horizontalPadding: selectData<"" | "sm" | "md" | "lg" | "xl">({
          title() {
            return "Horizontal Padding";
          },
          options: horizontalPadding as any,
        }),
      },
    };
  },
  import.meta.hot,
);

export default MyComponent;
export const passive = true;
export const id = "oyf5ut4svjmudc6kl6hhm69h";
