import { createBlueprint, BlueprintComponent } from "nitropage";
import { css, rgb } from "nitropage/css";
import { colorData } from "nitropage/data/color";
import { numberData } from "nitropage/data/number";
import { textData } from "nitropage/data/text";
import { Show } from "solid-js";

const MyComponent: BlueprintComponent<typeof blueprint> = function (props) {
  const id = () => props.elementProps.id || "_" + props.element.id;

  return (
    <div
      class={css({
        "--c1": rgb(props.data.color1) || "transparent",
        "--c2": rgb(props.data.color2) || "transparent",
        "--c3": rgb(props.data.color3) || "transparent",
        "--v1":
          props.data.length1 != null ? props.data.length1 + "rem" : undefined,
        "--v2":
          props.data.length2 != null ? props.data.length2 + "rem" : undefined,
        "--v3":
          props.data.length3 != null ? props.data.length3 + "rem" : undefined,
        "--v4":
          props.data.length4 != null ? props.data.length4 + "rem" : undefined,
        "--c": "var(--c1)",
        "--s": "var(--v1)",
        padding: props.lax ? undefined : "1rem 0",
        "min-height": "3rem",
      })}
      style={props.data.style}
      {...props.elementProps}
      id={id()}
    >
      {props.handles}
      <Show when={props.data.css}>
        <style>
          {props.data.css
            .replaceAll("#element", "#" + id())
            .replaceAll("#slot", "#" + id() + ">.slot")}
        </style>
      </Show>
      <props.Slot
        class="slot flex flex-col gap-4"
        style={props.data.slotStyle}
        name="default"
      />
    </div>
  );
};

export const blueprint = createBlueprint(
  () => {
    const numberCommons = {
      optional: true,
      step: 0.125,
      max: 24,
    };

    return {
      slots: { default: {} },
      data: {
        style: textData({
          multiline: true,
        }),
        slotStyle: textData({
          multiline: true,
        }),
        css: textData({
          multiline: true,
          title: () => "CSS #element / #slot",
        }),
        color1: colorData({
          title: () => "Color1 var(--c1)",
        }),
        color2: colorData({
          title: () => "Color2 var(--c2)",
        }),
        color3: colorData({
          title: () => "Color3 var(--c3)",
        }),
        length1: numberData({
          ...numberCommons,
          title: () => "Length1 var(--v1)",
        }),
        length2: numberData({
          ...numberCommons,
          title: () => "Length2 var(--v2)",
        }),
        length3: numberData({
          ...numberCommons,
          title: () => "Length3 var(--v3)",
        }),
        length4: numberData({
          ...numberCommons,
          title: () => "Length4 var(--v4)",
        }),
      },
    };
  },
  import.meta.hot,
);

export default MyComponent;
export const passive = true;
export const id = "vigkf7auegf887geschdgb7w";
