import clsx from "clsx";
import { createBlueprint, BlueprintComponent } from "nitropage";
import { selectData } from "nitropage/data/select";

const MyComponent: BlueprintComponent<typeof blueprint> = function (props) {
  return (
    <div {...props.elementProps}>
      {props.handles}
      <props.Slot
        direction="horizontal"
        class={clsx(
          "flex items-center",
          props.data.verticalAlign === "" && "justify-center",
          props.data.verticalAlign === "right" && "justify-end",
          props.lax && "flex-wrap",
          props.data.spacing === "sm" && "gap-3",
          props.data.spacing === "md" && "gap-y-5 gap-x-6",
          props.data.spacing === "lg" && "gap-y-6 gap-x-12",
          props.data.spacing === "xl" && "gap-y-10 gap-x-20",
          !props.lax && "py-4",
        )}
        name="default"
      />
    </div>
  );
};

export const blueprint = createBlueprint(
  () => {
    return {
      acceptParentType: "element",
      slots: { default: {} },
      data: {
        verticalAlign: selectData<"" | "left" | "right">({
          options: [
            {
              title() {
                return "Center";
              },
            },
            {
              title() {
                return "Left";
              },
              value: "left",
            },
            {
              title() {
                return "Right";
              },
              value: "right",
            },
          ],
        }),
        spacing: selectData<"" | "sm" | "md" | "lg" | "xl">({
          options: [
            {
              title() {
                return "None";
              },
            },
            {
              title() {
                return "Small";
              },
              value: "sm",
            },
            {
              title() {
                return "Middle";
              },
              value: "md",
            },
            {
              title() {
                return "Large";
              },
              value: "lg",
            },
            {
              title() {
                return "Huge";
              },
              value: "xl",
            },
          ],
        }),
      },
    };
  },
  import.meta.hot,
);

export default MyComponent;
export const id = "cd3pv0e6xywtebw50ph4mc1s";
