import clsx from "clsx";
import { createBlueprint, BlueprintComponent } from "nitropage";
import { css } from "nitropage/data/color";
import { selectData, SelectOptions } from "nitropage/data/select";

const MyComponent: BlueprintComponent<typeof blueprint> = function (props) {
  return (
    <div class={clsx(props.admin && "min-h-[20px]")} {...props.elementProps}>
      {props.handles}
      <props.Slot
        optional
        name="header"
        class={clsx(
          props.data.headerSpacing === "sm" && "mb-5 lg:mb-9",
          props.data.headerSpacing === "md" && "mb-12 lg:mb-14",
          props.data.headerSpacing === "lg" && "mb-24 lg:mb-28",
        )}
      />
      <props.Slot
        class={clsx(
          "flex flex-col",
          props.data.itemsSpacing === "sm" && "gap-4 lg:gap-8",
          props.data.itemsSpacing === "md" && "gap-10 lg:gap-12",
          props.data.itemsSpacing === "lg" && "gap-20 lg:gap-24",
          props.data.itemsSpacing === "xl" && "gap-32 lg:gap-36",
        )}
        name="default"
      />
    </div>
  );
};

export const blueprint = createBlueprint(
  async ({ admin }) => {
    const lang = admin ? (await import("./space.lang")).default : ({} as never);

    type Spacings = "" | "sm" | "md" | "lg";
    const commonOptions: SelectOptions<Spacings> = [
      { title: lang.none },
      { title: lang.small, value: "sm" },
      { title: lang.middle, value: "md" },
      { title: lang.large, value: "lg" },
    ];

    return {
      acceptParentType: "element",
      slots: { header: {}, default: {} },
      data: {
        itemsSpacing: selectData<Spacings | "xl">({
          title: lang.itemsSpacing,
          options: [...commonOptions, { title: lang.huge, value: "xl" }],
        }),
        headerSpacing: selectData<Spacings>({
          title: lang.headerSpacing,
          options: commonOptions,
        }),
      },
    };
  },
  import.meta.hot,
);

export default MyComponent;
export const id = "ktfqyfx6i7gru55hgndh0d9h";
