import {
  ComponentProps,
  FlowProps,
  ValidComponent,
  splitProps,
} from "solid-js";
import { Dynamic } from "solid-js/web";
import { Button as KButton } from "@kobalte/core";
import clsx from "clsx";
import { css } from "nitropage/css";

const Button = function <
  T extends ValidComponent = "button",
  P extends ComponentProps<T> = ComponentProps<T>,
>(
  props: FlowProps<{
    textColor?: (string | undefined)[];
    bgColor?: (string | undefined)[];
    active?: boolean;
    noActiveStyle?: boolean;
    clickScale?: boolean;
    class?: string;
    component?: T;
    size?: "" | "sm" | "lg";
  }> & { [K in keyof P]: P[K] },
) {
  const [local, rest] = splitProps(props, [
    "clickScale",
    "textColor",
    "bgColor",
    "active",
    "size",
    "component",
    "class",
    "children",
  ]);
  return (
    <Dynamic
      component={local.component || (KButton.Root as any)}
      class={clsx(
        local.class,
        css({
          color: props.textColor?.[0],
          backgroundColor: props.bgColor?.[0],
          [":hover,&.active" + (props.noActiveStyle ? "-def" : "")]: {
            color: props.textColor?.[1],
            backgroundColor: props.bgColor?.[1],
          },
        }),
        "leading-none inline-flex items-center justify-center text-white hover:text-white bg-slate-700 hover:bg-slate-900 transition-all ease-out text-center cursor-pointer ",
        local.clickScale !== false && "active:scale-105",
        local.active && "active scale-110 shadow-lg bg-slate-900",
        local.size === "sm" &&
          "py-2 px-3 lg:px-5 text-xs md:text-sm gap-2 rounded shadow",
        (!local.size || local.size === "") &&
          "py-2 px-5 lg:px-7 text-base md:text-lg gap-2 rounded-md shadow",
        local.size === "lg" &&
          "py-3 px-8 lg:px-12 text-lg md:text-xl gap-3 rounded-md shadow-md",
      )}
      {...rest}
    >
      {local.children}
    </Dynamic>
  );
};

export default Button;
