import { FlowComponent, JSX, Show, children, splitProps } from "solid-js";
import { Dynamic } from "solid-js/web";
import clsx from "clsx";
import { css } from "nitropage/css";

const Heading: FlowComponent<{
  class?: string;
  fullWidth?: boolean;
  level: number;
  size?: "" | "sm" | "md" | "lg" | "xl";
  subtitle?: JSX.Element;
  subtitleGap?: number;
  prependSubtitle?: boolean;
}> = function (props) {
  const [local, rest] = splitProps(props, [
    "fullWidth",
    "children",
    "class",
    "level",
    "size",
    "subtitle",
    "prependSubtitle",
  ]);
  const child = children(() => local.children);
  const subtitle = children(() => local.subtitle);

  return (
    <Show when={child()}>
      <div
        class={clsx(
          local.class,
          props.subtitleGap != null && css({ gap: props.subtitleGap + "em" }),
          "heading flex gap-[0.1em] flex-col font-bold text-slate-800 break-words lg:break-normal",
          props.prependSubtitle && "flex-col-reverse",
          !local.size && "sm:text-lg lg:text-xl",
          local.size === "sm" && "text-xl sm:text-2xl lg:text-3xl",
          local.size === "md" && "text-2xl sm:text-3xl lg:text-4xl",
          local.size === "lg" && "text-3xl sm:text-4xl lg:text-5xl",
          local.size === "xl" && "text-4xl sm:text-5xl lg:text-6xl",
        )}
        {...rest}
      >
        <div class={clsx(!props.fullWidth && "md:w-5/6")}>
          <Dynamic component={"h" + local.level}>{child()}</Dynamic>
        </div>
        <Show when={subtitle()}>
          <div
            class={clsx(
              "heading__subtitle text-slate-500 font-semibold leading-tight",
              !props.fullWidth && "md:w-4/6",
              !local.size && "text-[0.9em]",
              local.size === "sm" && "text-[0.8em]",
              local.size === "md" && "text-[0.7em]",
              !!(local.size && ["lg", "xl"].includes(local.size)) &&
                "text-[0.6em]",
            )}
          >
            {subtitle()}
          </div>
        </Show>
      </div>
    </Show>
  );
};

export default Heading;
