import { FlowComponent } from "solid-js";
import { createEffect, createSignal } from "solid-js";
import { Dialog } from "@kobalte/core";
import { Image, ImageContainer } from "nitropage/data/media";
import classes from "./imageMagnifier.module.css";

const ImageMagnifier: FlowComponent<{
  width?: number;
  height?: number;
  url: string;
}> = function (props) {
  const [open, setOpen] = createSignal(false);

  const close = () => setOpen(false);
  createEffect(function () {
    if (!open()) {
      window.removeEventListener("scroll", close);
      return;
    }

    window.addEventListener("scroll", close);
  });

  return (
    <Dialog.Root modal={false} open={open()} onOpenChange={setOpen}>
      <Dialog.Trigger aria-label="Image Zoom" class="w-full cursor-zoom-in">
        {props.children}
      </Dialog.Trigger>
      <Dialog.Portal>
        <Dialog.Overlay
          class={
            "fixed z-50 inset-0 bg-white/30 backdrop-blur-md " +
            classes["overlay"]
          }
          onClick={() => setOpen(false)}
        />
        <Dialog.Content class={"z-50 fixed inset-0 " + classes["content"]}>
          <Dialog.CloseButton class="cursor-zoom-out outline-none w-full h-full p-8 lg:p-14 flex items-center justify-center">
            <ImageContainer fit={"contain"}>
              <Image
                sources={[
                  {
                    url: props.url,
                    width: props.width,
                    height: props.height,
                  },
                ]}
              />
            </ImageContainer>
          </Dialog.CloseButton>
        </Dialog.Content>
      </Dialog.Portal>
    </Dialog.Root>
  );
};

export default ImageMagnifier;
