import { colorData } from "nitropage/data/color";

type Keys = Record<string, "text" | "textHover" | "bg" | "bgHover">;
export const commonKeys = {
  textColor: "text",
  textColorHover: "textHover",
  bgColor: "bg",
  bgColorHover: "bgHover",
} satisfies Keys;

export const prefixedKeys = {
  buttonTextColor: "text",
  buttonTextColorHover: "textHover",
  buttonBgColor: "bg",
  buttonBgColorHover: "bgHover",
} satisfies Keys;

export const buttonColorData = function <const T extends Keys>({
  title,
  keys,
}: {
  title?: string;
  keys: T;
}) {
  const title_ = title ? `${title} ` : "";

  type Result = { [K in keyof T]: ReturnType<typeof colorData> };
  const result = {} as Result;

  for (const [k, v] of Object.entries(keys)) {
    result[k as keyof T] = colorData({
      title() {
        const suffix = v.startsWith("bg") ? "Background" : "Text";
        const hover = v.endsWith("Hover") ? " Hover" : "";
        return `${title_}${suffix}-Color ${hover}`;
      },
    });
  }

  return result;
};
